@@ -41,7 +41,7 @@ static char sccsid[] = "@(#)glob.c	5.7 (Berkeley) 12/14/88";
 #include <errno.h>
 #include <pwd.h>
 
-#if !defined(dirfd) && !defined(__GLIBC__) && !defined(__linux__) && !defined(__FreeBSD__) && !defined(__CYGWIN__)
+#if !defined(dirfd) && !defined(__GLIBC__) && !defined(__linux__) && !defined(__FreeBSD__) && !defined(__CYGWIN__) && !defined(__APPLE__)
 #define dirfd(dirp)  ((dirp)->dd_fd)
 #endif
 
@@ -71,7 +71,9 @@ static  void matchdir(char *);
 static  int  execbrc(char *, char *);
 static  int  match(char *, char *);
 static  int  amatch(char *, char *);
+#if 0
 static  int  Gmatch(char *, char *);
+#endif
 static  void Gcat(char *, char *);
 static  void addpath(char);
 static  void fatal(char *);
@@ -474,6 +476,7 @@ slash:
 	}
 }
 
+#if 0
 static int
 Gmatch(s, p)
 	char *s, *p;
@@ -535,6 +538,7 @@ Gmatch(s, p)
 		}
 	}
 }
+#endif
 
 static void
 Gcat(s1, s2)
