@@ -93,6 +93,7 @@ void UIloadinterface()
   int		n, oldstderr;
   int		argcblah = 1;
   char 	       *argvblah[1] = { "Animate" };
+  FILE         *dummy;
 
   mib_Widget   *MainForm, *InfoForm, *CommandForm;
   String	fallbacks[] = {
@@ -124,7 +125,8 @@ void UIloadinterface()
   dpy = XtDisplay(TopLevel);
 
   oldstderr = dup(fileno(stderr));
-  freopen("/dev/null", "w", stderr);
+  dummy = freopen("/dev/null", "w", stderr);
+  (void)dummy;
 
   /* configure resize policy of window */
 
@@ -321,7 +323,7 @@ void UIloadinterface()
 		(XtPointer)NULL
 		);
 
-  freopen("/dev/null", "w", stderr);
+  dummy = freopen("/dev/null", "w", stderr);
   dup2(oldstderr, fileno(stderr));
   close(oldstderr);
 
