@@ -518,14 +518,14 @@ void gv_pick_invisible(int a1)
   LFree(val);
 }
 
-void gv_rawevent(int a1, int a2, int a3, int a4, unsigned long a5)
+void gv_rawevent(int a1, int a2, int a3, int a4, long a5)
 {
   LObject *val = LEvalFunc("rawevent",
 			   LINT, a1,
 			   LINT, a2,
 			   LINT, a3,
 			   LINT, a4,
-			   LULONG, a5,
+			   LLONG, a5,
 			   LEND);
   LFree(val);
 }
@@ -635,6 +635,13 @@ void gv_dump_handles()
   LFree(val);
 }
 
+void gv_dump_pools()
+{
+  LObject *val = LEvalFunc("dump-pools",
+			   LEND);
+  LFree(val);
+}
+
 char * gv_geomview_version()
 {
   LObject *val = LEvalFunc("geomview-version",
@@ -1076,6 +1083,13 @@ char * gv_bgimagefile(int a1, char * a2)
   return retval;
 }
 
+void gv_processevents()
+{
+  LObject *val = LEvalFunc("process-events",
+			   LEND);
+  LFree(val);
+}
+
 void gv_ui_motion(int a1, int a2)
 {
   LObject *val = LEvalFunc("ui-motion",
@@ -1230,6 +1244,7 @@ extern LObject *Lshell(Lake *, LList *);
 extern LObject *Lwrite_sexpr(Lake *, LList *);
 extern LObject *Lwrite_handle(Lake *, LList *);
 extern LObject *Ldump_handles(Lake *, LList *);
+extern LObject *Ldump_pools(Lake *, LList *);
 extern LObject *Lgeomview_version(Lake *, LList *);
 extern LObject *Lrib_display(Lake *, LList *);
 extern LObject *Lrib_snapshot(Lake *, LList *);
@@ -1272,6 +1287,7 @@ extern LObject *Lcamera_prop(Lake *, LList *);
 extern LObject *Lwrite(Lake *, LList *);
 extern LObject *Lsnapshot(Lake *, LList *);
 extern LObject *Lbgimagefile(Lake *, LList *);
+extern LObject *Lprocessevents(Lake *, LList *);
 extern LObject *Lui_motion(Lake *, LList *);
 extern LObject *Lui_cam_focus(Lake *, LList *);
 extern LObject *Lui_target(Lake *, LList *);
@@ -1352,6 +1368,7 @@ extern char Hshell[];
 extern char Hwrite_sexpr[];
 extern char Hwrite_handle[];
 extern char Hdump_handles[];
+extern char Hdump_pools[];
 extern char Hgeomview_version[];
 extern char Hrib_display[];
 extern char Hrib_snapshot[];
@@ -1394,6 +1411,7 @@ extern char Hcamera_prop[];
 extern char Hwrite[];
 extern char Hsnapshot[];
 extern char Hbgimagefile[];
+extern char Hprocessevents[];
 extern char Hui_motion[];
 extern char Hui_cam_focus[];
 extern char Hui_target[];
@@ -1477,6 +1495,7 @@ void clang_init()
   LDefun("write-sexpr", Lwrite_sexpr, Hwrite_sexpr);
   LDefun("write-handle", Lwrite_handle, Hwrite_handle);
   LDefun("dump-handles", Ldump_handles, Hdump_handles);
+  LDefun("dump-pools", Ldump_pools, Hdump_pools);
   LDefun("geomview-version", Lgeomview_version, Hgeomview_version);
   LDefun("rib-display", Lrib_display, Hrib_display);
   LDefun("rib-snapshot", Lrib_snapshot, Hrib_snapshot);
@@ -1519,6 +1538,7 @@ void clang_init()
   LDefun("write", Lwrite, Hwrite);
   LDefun("snapshot", Lsnapshot, Hsnapshot);
   LDefun("background-image", Lbgimagefile, Hbgimagefile);
+  LDefun("process-events", Lprocessevents, Hprocessevents);
   LDefun("ui-motion", Lui_motion, Hui_motion);
   LDefun("ui-cam-focus", Lui_cam_focus, Hui_cam_focus);
   LDefun("ui-target", Lui_target, Hui_target);
