@@ -108,6 +108,15 @@ static HandleOps listenOps = {	/* "Ops" structure for listening sockets */
 };
 #endif
 
+static int ign_read(int fd, void *buffer, size_t size)
+{
+  int result;
+  
+  result = read(fd, buffer, size);
+  
+  return result;
+}
+
 static int 
 commandimport(Pool *p, Handle **unused, Ref **unused_too )
 {
@@ -276,23 +285,26 @@ comm_object(char *str, HandleOps *ops, Handle **hp, Ref **rp, int now)
 
 /*
  * Arrange that later attempts to read the same file will actually re-read it.
+ *
+ * cH: above comment is crap: at least since v1.6 a PoolClose() would
+ * result in actually closing the file, so a new PoolStreamOpen()
+ * would position right at the start of the data. We just delete the
+ * pool.
  */
 static int
 commandclose(Pool *p)
 {
-  PoolDoReread(p);
   PoolClose(p);
-#if 1 /* should we really cache handles for command-files??? */
-  PoolDelete(p);
-#endif
+  MyPoolDelete(p);
   return 0;
 }
 
 
 LDEFINE(setenv, LVOID,
-	"(setenv  name string)  sets the environment variable ``name'' to the value\n\
-``string''; the name is visible to geomview (as in pathnames containing $name)\n\
-and to processes it creates, e.g. external modules.")
+	"(setenv  name string)\n"
+	"Sets the environment variable ``name'' to the value ``string''; the "
+	"name is visible to geomview (as in pathnames containing $name) "
+	"and to processes it creates, e.g. external modules.")
 {
   char *name, *string = NULL;
   char buf[10240];
@@ -309,12 +321,13 @@ and to processes it creates, e.g. external modules.")
 
 
 LDEFINE(input_translator, LVOID,
-	"(input-translator  \"#prefix_string\"  \"Bourne-shell-command\")\n\
-Defines an external translation program for special input types.\n\
-When asked to read a file which begins with the specified string,\n\
-geomview invokes that program with standard input coming from the given file.\n\
-The program is expected to emit OOGL geometric data to its standard output.\n\
-In this implementation, only prefixes beginning with # are recognized.")
+	"(input-translator  \"#prefix_string\"  \"Bourne-shell-command\")\n"
+	"Defines an external translation program for special input types. "
+	"When asked to read a file which begins with the specified string, "
+	"Geomview invokes that program with standard input coming from the "
+	"given file. The program is expected to emit OOGL geometric data to "
+	"its standard output. In this implementation, only prefixes beginning "
+	"with # are recognized.")
 {
   char *prefix, *cmd;
   LDECLARE(("input-translator", LBEGIN,
@@ -417,7 +430,7 @@ fatalsig(int sig)
 	  "dump core now (y/n) [n] ? ",
 	  getpid(), signal_name(sig));
   fflush(stderr);
-  read(2, &die, 1);
+  ign_read(2, &die, 1);
   fprintf(stderr, "got answer %c\n", (int)die % 0xff);
   fflush(stderr);
   if (die != 'y' && die != 'Y')
@@ -537,6 +550,7 @@ LDEFINE(emodule_run, LVOID,
   LDECLARE(("emodule-run", LBEGIN,
 	    LSTRINGS, &cmd,
 	    LEND));
+
   em.text = em.name = cmd;
   em.data = NULL;
   em.dir = NULL;
@@ -596,6 +610,7 @@ LDEFINE(emodule_isrunning, LVOID,
   LDECLARE(("emodule-isrunning", LBEGIN,
 	    LSTRING, &modname,
 	    LEND));
+
   if (emodule_running_index(modname) != -1) return Lt;
   else return Lnil;
 }
@@ -662,7 +677,6 @@ emodule_run(emodule *em)
     return NULL;
 
   case 0: {
-    static char rats[] = "Can't exec external module: ";
     char envbuf[10240];
 
     if (otherpgrp) {
@@ -701,7 +715,7 @@ emodule_run(emodule *em)
     signal(SIGCHLD, SIG_DFL);
     execl("/bin/sh", "sh", "-c", program, NULL);
 
-    write(STDERR_FILENO, rats, sizeof(rats)-1);
+    fprintf(stderr, "Can't exec external module: ");
     perror(em->text);
     exit(1);
   }
@@ -756,11 +770,13 @@ LDEFINE(command, LVOID,
 	by default.")
 {
   char *file, *ofile = NULL;
+
   LDECLARE(("command", LBEGIN,
 	    LSTRING, &file,
 	    LOPTIONAL,
 	    LSTRING, &ofile,
 	    LEND));
+
   if (PoolStreamOpen(file, NULL, 0, &CommandOps) == NULL) {
     OOGLError(0,"command: cannot open input %s: %s\n", file, sperror());
     return Lnil;
@@ -790,6 +806,7 @@ LDEFINE(sleep_for, LVOID,
 	    LLAKE, &sweenie,
 	    LFLOAT, &time,
 	    LEND));
+
   PoolSleepFor(POOL(sweenie), time);
   return Lt;
 }
@@ -811,6 +828,7 @@ LDEFINE(sleep_until, LFLOAT,
 	    LLAKE, &bass,
 	    LFLOAT, &time,
 	    LEND));
+
   PoolSleepUntil(POOL(bass), time);
   time -= PoolTimeAt(POOL(bass), NULL);	/* NULL => now. */
   return LNew(LFLOAT, &time);
@@ -823,18 +841,21 @@ LDEFINE(set_clock, LVOID,
 {
   Lake *bass;
   float time;
+
   LDECLARE(("set-clock", LBEGIN,
 	    LLAKE, &bass,
 	    LFLOAT, &time,
 	    LEND));
+
   PoolSetTime(POOL(bass), NULL, time);
+
   return Lt;
 }
 
 LDEFINE(clock, LVOID,
-	"(clock)\n\
-	Returns the current time, in seconds, as shown by this stream's clock.\n\
-	See also set-clock and sleep-until.")
+	"(clock)\n"
+	"Returns the current time, in seconds, as shown by this stream's "
+	"clock.	See also set-clock and sleep-until.")
 {
   Lake *rainy;
   float time;
@@ -857,16 +878,25 @@ void echo_to_fp(LList *arglist, FILE *fp)
   } else {
     for(;;) {
       arg = arglist->car;
-      if (arg->type == LSTRING)
+      /* Note that -- for compatibility -- we distinguish the case of
+       * a function which evaluates to a string and a literal
+       * string. A literal string is written without quotes, a
+       * function evaluating to a string results in printing a quoted
+       * string. An example is (echo (read-id focus)) which used to
+       * result in printing the id of the active camera as quoted
+       * string.
+       */
+      if (arg->type == LSTRING) {
+	/* omit the quotes */
 	fputs(LSTRINGVAL(arg), fp);
-      else if (arg->type == LLIST) {
+      } else {
 	val = LEval(arg);
 	LWrite(fp, val);
 	LFree(val);
-      } else
-	LWrite(fp, arg);
-      if ((arglist = arglist->cdr) == NULL)
+      }
+      if ((arglist = arglist->cdr) == NULL) {
 	break;
+      }
       fputs(" ", fp);
     }
   }
@@ -874,23 +904,26 @@ void echo_to_fp(LList *arglist, FILE *fp)
 }
 
 LDEFINE(echo, LVOID,
-	"(echo          ...)\n\
-	Write the given data to the special file \"-\".  Strings are written\n\
-	literally; lisp expressions are evaluated and their values written.\n\
-	If received from an external program, \"echo\" sends to the program's\n\
-	input.  Otherwise writes to geomview's own standard output\n\
-	(typically the terminal).")
+	"(echo ...)\n"
+	"Write the given data to the special file \"-\".  Strings are written "
+	"literally; lisp expressions are evaluated and their values written. "
+	"If received from an external program, \"echo\" sends to the "
+	"program's input. Otherwise writes to geomview's own standard output "
+	"(typically the terminal).")
 {
   Lake *powderhorn;
   LList *arglist = NULL;
   FILE *fp;
+
   LDECLARE(("echo", LBEGIN,
-	    LHOLD,
 	    LLAKE, &powderhorn,
-	    LREST, &arglist));
+	    LHOLD,
+	    LREST, &arglist,
+	    LEND));
   
-  if ( (fp = PoolOutputFile(POOL(powderhorn))) == NULL)
+  if ((fp = PoolOutputFile(POOL(powderhorn))) == NULL) {
     fp = stdout;
+  }
   echo_to_fp(arglist, fp);
 
   return Lt;
@@ -909,13 +942,17 @@ LDEFINE(emodule_transmit, LVOID,
   emodule *em;
   LList *message = NULL;
   int i;
+
   LDECLARE(("emodule-transmit", LBEGIN,
 	    LSTRING, &modname,
 	    LHOLD,
-	    LREST, &message));
+	    LREST, &message,
+	    LEND));
 
   i = emodule_running_index(modname);
-  if (i == -1) return Lnil;
+  if (i == -1) {
+    return Lnil;
+  }
   em = VVINDEX(uistate.emod, emodule, i);
   if (em->link && em->link->outf) {
     echo_to_fp(message, em->link->outf);
@@ -927,7 +964,8 @@ LDEFINE(emodule_transmit, LVOID,
 
 
 LDEFINE(read, LVOID,
-	"(read {geometry|camera|image|appearance|transform|ntransform|command} {GEOMETRY or CAMERA or ...})\n"
+	"(read {geometry|camera|image|appearance|transform|ntransform|command} "
+	"{GEOMETRY or CAMERA or ...})\n"
 	"Read and interpret the text in ... as containing the "
 	"given type of data.  Useful for defining objects using OOGL "
 	"reference syntax, e.g. "
@@ -948,15 +986,18 @@ LDEFINE(read, LVOID,
   char *opsname = NULL;
   HandleOps *ops;
   LObject *kw = NULL;
-  int c;
 
-  if (lake != NULL) {
+  if (LPARSEMODE) {
     /* parse first arg [ops]: */
-    if (!LakeMore(lake, c) ||
+    if (!LakeMore(lake) ||
 	(kw = LSexpr(lake)) == Lnil ||
-	!LFROMOBJ(LSTRING)(kw, &opsname) ||
+	!LSTRINGFROMOBJ(kw, &opsname) ||
 	!(ops = str2ops(opsname))) {
-      OOGLSyntax(lake->streamin, "\"read\" in \"%s\": keyword expected {command|geometry|camera|window|transform|ntransform|image|appearance}, got \"%s\"",
+      OOGLSyntax(lake->streamin,
+		 "\"read\" in \"%s\": keyword "
+		 "{command|geometry|camera|window|transform|"
+		 "ntransform|image|appearance} expected, "
+		 "got \"%s\"",
 		 LakeName(lake), opsname);
       goto fail;
     }
@@ -964,7 +1005,7 @@ LDEFINE(read, LVOID,
     /* parse 2nd arg, using ops determined by 1st arg.
        Note: we don't actually store the 1st arg because this function's
        work is all done during parsing.  */
-    if (!LakeMore(lake,c) || (*ops->strmin)(POOL(lake), NULL, NULL) == 0) {
+    if (!LakeMore(lake) || (*ops->strmin)(POOL(lake), NULL, NULL) == 0) {
       OOGLSyntax(lake->streamin, "\"read %s\" in \"%s\": error reading %s's",
 		 opsname, PoolName(POOL(lake)), opsname);
       goto fail;
@@ -974,6 +1015,7 @@ LDEFINE(read, LVOID,
      no evaluation work; it's all in the parsing */
   LFree(kw);
   return Lt;
+
  fail:
   LFree(kw);
   return Lnil;
@@ -988,20 +1030,20 @@ void gv_merge(HandleOps *ops, int camid, Ref *object)
   cs.cam = REFGET(Camera, object);     /* Since (merge ...) will delete it */
 #endif
 
-  LFree( LEvalFunc("merge",
-		   LSTRING, ops == &CamOps ? "camera" : "window",
-		   LID, camid,
-		   ops == &CamOps ? LCAMERA : LWINDOW, &cs,
-		   LEND) );
+  LFree(LEvalFunc("merge",
+		  LSTRING, ops == &CamOps ? "camera" : "window",
+		  LID, camid,
+		  ops == &CamOps ? LCAMERA : LWINDOW, &cs,
+		  LEND) );
 }
 
 LDEFINE(merge, LVOID,
-	"(merge          {window|camera} CAM-ID  { WINDOW or CAMERA ... } )\n\
-	Modify the given window or camera, changing just those properties\n\
-	specified in the last argument.  E.g.\n\
-		(merge camera \"Camera\" { far 20 })\n\
-	sets Camera's far clipping plane to 20 while leaving\n\
-	other attributes untouched.")
+	"(merge {window|camera} CAM-ID  { WINDOW or CAMERA ... } )\n"
+	"Modify the given window or camera, changing just those properties "
+	"specified in the last argument. E.g.\n\n"
+	"               (merge camera \"Camera\" { far 20 })\n\n"
+	"sets Camera's far clipping plane to 20 while leaving "
+	"other attributes untouched.")
 /*
   Since we don't use LDECLARE we must remember to LDefun()
   this function manually in lang_init() in file lang.c.
@@ -1009,21 +1051,22 @@ LDEFINE(merge, LVOID,
 {
   char *opsname = NULL;
   HandleOps *ops;
-  int c, id;
+  int id;
   LObject *kw = NULL, *idarg = NULL, *item = NULL;
 
-  if (lake != NULL) {
+  if (LPARSEMODE) {
     /* parse first arg [ops]: */
-    if (! LakeMore(lake,c) || (kw = LSexpr(lake)) == Lnil ||
-	!LFROMOBJ(LSTRING)(kw, &opsname) ||
+    if (!LakeMore(lake) || (kw = LSexpr(lake)) == Lnil ||
+	!LSTRINGFROMOBJ(kw, &opsname) ||
 	((ops = str2ops(opsname)) != &CamOps && ops != &WindowOps)) {
       OOGLSyntax(lake->streamin,
-		 "\"merge\" in \"%s\": expected \"camera\" or \"window\", got \"%s\"", LakeName(lake), opsname);
+		 "\"merge\" in \"%s\": expected \"camera\" or \"window\", "
+		 "got \"%s\"", LakeName(lake), opsname);
       goto parsefail;
     }
 
     /* parse 2nd arg; it's a string (id) */
-    if (! LakeMore(lake,c) || (idarg = LEvalSexpr(lake)) == Lnil) {
+    if (!LakeMore(lake) || (idarg = LEvalSexpr(lake)) == Lnil) {
       OOGLSyntax(lake->streamin,"\"merge\" in \"%s\": expected CAM-ID",
 		 LakeName(lake));
       goto parsefail;
@@ -1039,13 +1082,15 @@ LDEFINE(merge, LVOID,
     LListAppend(args, idarg);
     LListAppend(args, item);
     return Lt;
+  } else {
+    LParseArgs("merge", LBEGIN, LREST, &args, LEND);
   }
 
   kw = LListEntry(args, 1);
   idarg = LListEntry(args, 2);
   item = LListEntry(args, 3);
-  if (!LFROMOBJ(LSTRING)(kw, &opsname) ||
-     ((ops = str2ops(opsname)) != &CamOps && ops != &WindowOps)) {
+  if (!LSTRINGFROMOBJ(kw, &opsname) ||
+      ((ops = str2ops(opsname)) != &CamOps && ops != &WindowOps)) {
     OOGLError(0, "\"merge\": expected \"camera\" or \"window\", got %s",
 	      LSummarize(idarg));
     return Lnil;
@@ -1063,8 +1108,7 @@ LDEFINE(merge, LVOID,
     }
     drawer_merge_camera(id, cs->cam);
     /* CamDelete(cs->cam); */
-  }
-  else {
+  } else {
     WindowStruct *ws;
     if (!LFROMOBJ(LWINDOW)(item, &ws)) {
       OOGLError(0,"\"merge\": expected window in arg position 3");
@@ -1258,13 +1302,15 @@ LDEFINE(hdelete, LVOID,
 
 
 LDEFINE(hdefine, LVOID,
-	"(hdefine {geometry|camera|window|image|appearance|transform|ntransform} name value)\n"
-	"Sets the value of a handle of a given type."
-	"(hdefine  <type>  <name>  <value>)  is generally equivalent to"
-	"(read <type>  { define <name> <value> })"
-	"except that the assignment is done when hdefine is executed,"
-	"(possibly not at all if inside a conditional statement),"
-	"while the ``read ... define'' performs assignment as soon as the"
+	"(hdefine "
+	"{geometry|camera|window|image|appearance|transform|ntransform} "
+	"name value)\n"
+	"Sets the value of a handle of a given type. "
+	"(hdefine  <type>  <name>  <value>)  is generally equivalent to "
+	"(read <type>  { define <name> <value> }) "
+	"except that the assignment is done when hdefine is executed, "
+	"(possibly not at all if inside a conditional statement), "
+	"while the ``read ... define'' performs assignment as soon as the "
 	"text is read.")
 {
   Handle *h = NULL;
@@ -1273,7 +1319,6 @@ LDEFINE(hdefine, LVOID,
   char *hname;
   char *opsname = NULL;
   Ref *obj;
-  int c;
   LObject *kw = NULL, *name = NULL, *item = NULL;
   union {
     GeomStruct gs;
@@ -1286,10 +1331,10 @@ LDEFINE(hdefine, LVOID,
     LObject lobj;
   } *s;
 
-  if (lake != NULL) {
+  if (LPARSEMODE) {
     /* parse first arg [ops]: */
-    if (! LakeMore(lake,c) || (kw = LSexpr(lake)) == Lnil ||
-	!LFROMOBJ(LSTRING)(kw, &opsname) ||
+    if (! LakeMore(lake) || (kw = LSexpr(lake)) == Lnil ||
+	!LSTRINGFROMOBJ(kw, &opsname) ||
 	(ops = str2ops(opsname)) == NULL ||
 	(ltype = ops2ltype(ops)) == NULL) {
       OOGLSyntax(lake->streamin,
@@ -1302,7 +1347,7 @@ LDEFINE(hdefine, LVOID,
     }
 
     /* parse 2nd arg; it's a string (id) */
-    if (! LakeMore(lake,c) || (name = LEvalSexpr(lake)) == Lnil) {
+    if (!LakeMore(lake) || (name = LEvalSexpr(lake)) == Lnil) {
       OOGLSyntax(lake->streamin,
 		 "\"hdefine %s\" in \"%s\": expected handle name",
 		 LakeName(lake), opsname);
@@ -1319,20 +1364,23 @@ LDEFINE(hdefine, LVOID,
     LListAppend(args, kw);
     LListAppend(args, name);
     LListAppend(args, item);
+
     return Lt;
   }
 
+  LParseArgs("hdefine", LBEGIN, LREST, &args, LEND);
+
   kw = LListEntry(args, 1);
   name = LListEntry(args, 2);
   item = LListEntry(args, 3);
-  if (!LFROMOBJ(LSTRING)(kw, &opsname) ||
-     (ops = str2ops(opsname)) == NULL ||
-     (ltype = ops2ltype(ops)) == NULL) {
+  if (!LSTRINGFROMOBJ(kw, &opsname) ||
+      (ops = str2ops(opsname)) == NULL ||
+      (ltype = ops2ltype(ops)) == NULL) {
     OOGLError(0, "\"hdefine\": expected data type, got %s",
 	      LSummarize(kw));
     return Lnil;
   }
-  if (!LFROMOBJ(LSTRING)(name, &hname)) {
+  if (!LSTRINGFROMOBJ(name, &hname)) {
     OOGLError(0, "\"hdefine\": expected handle name, got %s", LSummarize(name));
     return Lnil;
   }
@@ -1354,7 +1402,6 @@ LDEFINE(hdefine, LVOID,
   if (ops == &TransOps) {
     REFPUT(obj); /* otherwise obj will never be deleted */
   }
-
   return Lt;
 
  parsefail:
@@ -1525,7 +1572,7 @@ usepipe(char *pipedir, char *suffix, char *pipetype)
     inetsocket = 3,
     inet6socket = 4
   } usepipe = nopipe;
-  int port;
+  int port = -1;
 
   while (*pipetype) switch(*pipetype++) {
   case 's':
@@ -1640,7 +1687,7 @@ usepipe(char *pipedir, char *suffix, char *pipetype)
  * Since (new_)geometry and (new_)camera take ownership of the object,
  * increasing its reference count, we decr the ref count after handing it over.
  */
-void
+static void
 useconnection(char *name, HandleOps *ops, Handle *h, Ref *obj, int unique)
 {
   char *tail;
@@ -1653,9 +1700,11 @@ useconnection(char *name, HandleOps *ops, Handle *h, Ref *obj, int unique)
   if (ops == &GeomOps) {
     GeomStruct gs;
     if(h && !h->permanent) {
+#if 0
       /* same logic as in geomparse() */
       HandleDelete(h);
       h = NULL;
+#endif
     }
     gs.h = h;
     gs.geom = (Geom *)obj;
@@ -1668,9 +1717,11 @@ useconnection(char *name, HandleOps *ops, Handle *h, Ref *obj, int unique)
   } else if (ops == &CamOps) {
     CameraStruct cs;
     if(h && !h->permanent) {
+#if 0
       /* same logic as in geomparse() */
       HandleDelete(h);
       h = NULL;
+#endif
     }
     cs.h = h;
     cs.cam = (Camera *)obj;
