@@ -53,9 +53,6 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include "cmodel.h"
 #include "mouse.h"
 #include <stdlib.h>
-#ifndef alloca		/* if no alloca in stdlib.h */
-#include <alloca.h>
-#endif
 #include <string.h>
 #include <sys/stat.h>
 #include <math.h>
@@ -443,6 +440,7 @@ drawer_id2name( int id )
   case DEFAULTCAMID:return "defaultcam";
   case TARGETID:   return "target";
   case CENTERID:   return "center";
+  case BBOXCENTERID: return "bbox-center";
   case PRIMITIVE:  return "primitive";
   case NOID:  return "none";
   default:
@@ -617,11 +615,11 @@ drawer_name2metaid(char *name)
 
 
 LDEFINE(name_object, LVOID,
-	"(name-object    ID NAME)\n\
-	Assign a new NAME (a string) to ID.  A number is appended if\n\
-	that name is in use (for example, \"foo\" -> \"foo<2>\").  The new\n\
-	name, possibly with number appended, may be used as object's\n\
-	id thereafter.")
+	"(name-object ID NAME)\n"
+	"Assign a new NAME (a string) to ID.  A number is appended if "
+	"that name is in use (for example, \"foo\" -> \"foo<2>\").  The new "
+	"name, possibly with number appended, may be used as object's "
+	"id thereafter.")
 {
   int id;
   char *name;
@@ -644,10 +642,10 @@ drawer_name_object(int id, int ni, char *name)
 }
 
 LDEFINE(update_draw, LVOID,
-	"(update-draw    CAM-ID  [timestep_in_seconds])\n\
-	Apply each incremental motion once and then draw CAM-ID.\n\
-	Applies \"timestep\" seconds' worth of motion, or uses elapsed real\n\
-	time if \"timestep\" is absent or zero.")
+	"(update-draw CAM-ID  [timestep_in_seconds])\n"
+	"Apply each incremental motion once and then draw CAM-ID. "
+	"Applies \"timestep\" seconds' worth of motion, or uses elapsed real "
+	"time if \"timestep\" is absent or zero.")
 {
   int id;
   float dt = 0;
@@ -661,8 +659,8 @@ LDEFINE(update_draw, LVOID,
 }
 
 LDEFINE(draw, LVOID,
-	"(draw           CAM-ID)\n\
-	Draw the view in CAM-ID, if it needs redrawing.  See also \"redraw\".")
+	"(draw CAM-ID)\n"
+	"Draw the view in CAM-ID, if it needs redrawing. See also \"redraw\".")
 {
   DView *dv;
   int index;
@@ -679,9 +677,9 @@ LDEFINE(draw, LVOID,
 }
 
 LDEFINE(update, LVOID,
-	"(update [timestep_in_seconds])\n\
-	Apply each incremental motion once.  Uses timestep if it's present and\n\
-	nonzero; otherwise motions are proportional to elapsed real time.")
+	"(update [timestep_in_seconds])\n"
+	"Apply each incremental motion once. Uses timestep if it's present and "
+	"nonzero; otherwise motions are proportional to elapsed real time.")
 {
   float realdt, dt = 0;
   LDECLARE(("update", LBEGIN,
@@ -709,9 +707,9 @@ drawer_updateproc(int id, PFI func)
 }
 
 LDEFINE(redraw, LVOID,
-	"(redraw         CAM-ID)\n\
-	States that the view in CAM-ID should be redrawn on the\n\
-	next pass through the main loop or the next invocation of \"draw\".")
+	"(redraw CAM-ID)\n"
+	"States that the view in CAM-ID should be redrawn on the "
+	"next pass through the main loop or the next invocation of \"draw\".")
 {
   DView *dv;
   int index;
@@ -729,18 +727,20 @@ LDEFINE(redraw, LVOID,
 }
 
 LDEFINE(freeze, LVOID,
-	"(freeze         CAM-ID  [hard-freeze])\n\
-	Freeze CAM-ID; drawing in this camera's window is turned off\n\
-	until it is explicitly redrawn with \"(redraw CAM-ID)\", after\n\
-	which time drawing resumes as normal.  hard-freeze (default false)\n\
-	is intended to be \"true\" only when windows are iconified,\n\
- 	making them immune even to \"redraw\" until another \"(freeze CAM-ID false)\".")
+	"(freeze         CAM-ID  [hard-freeze])\n"
+	"Freeze CAM-ID; drawing in this camera's window is turned off "
+	"until it is explicitly redrawn with \"(redraw CAM-ID)\", after "
+	"which time drawing resumes as normal.  hard-freeze (default false) "
+	"is intended to be \"true\" only when windows are iconified, "
+ 	"making them immune even to \"redraw\" until another "
+	"\"(freeze CAM-ID false)\".")
 {
   DView *dv;
   int index;
   int id;
   Keyword hard = NO_KEYWORD;
   int freeze;
+
   LDECLARE(("freeze", LBEGIN,
 	    LID, &id,
 	    LOPTIONAL,
@@ -804,9 +804,9 @@ void drawer_center(int id)
 
 
 LDEFINE(xform_set, LVOID,
-	"(xform-set      ID TRANSFORM)\n\
-	Overwrite the current object transform with TRANSFORM (set\n\
-	object ID's transform to TRANSFORM).")
+	"(xform-set ID TRANSFORM)\n"
+	"Overwrite the current object transform with TRANSFORM (set "
+	"object ID's transform to TRANSFORM).")
 {
   DObject *obj;
   int index;
@@ -848,11 +848,11 @@ LDEFINE(xform_set, LVOID,
 }
 
 LDEFINE(xform, LVOID,
-	"(xform          ID TRANSFORM)\n\
-	Apply TRANSFORM to object ID, as opposed to simply setting its\n\
-	transform, so the effective position\n\
-	of the object will be the concatenation of TRANSFORM with the\n\
-	current object transform.")
+	"(xform ID TRANSFORM)\n"
+	"Apply TRANSFORM to object ID, as opposed to simply setting its "
+	"transform, so the effective position "
+	"of the object will be the concatenation of TRANSFORM with the "
+	"current object transform.")
 {
   int id;
   TransformStruct *ts;
@@ -881,10 +881,10 @@ LDEFINE(xform, LVOID,
 }
 
 LDEFINE(xform_incr, LVOID,
-	"(xform-incr     ID TRANSFORM)\n\
-	Apply continual motion: concatenate TRANSFORM with the current\n\
-	transform of the object every refresh (set object ID's\n\
-	incremental transform to TRANSFORM).")
+	"(xform-incr     ID TRANSFORM)\n"
+	"Apply continual motion: concatenate TRANSFORM with the current "
+	"transform of the object every refresh (set object ID's "
+	"incremental transform to TRANSFORM).")
 {
   DObject *obj;
   int index;
@@ -915,10 +915,10 @@ LDEFINE(xform_incr, LVOID,
 }
 
 LDEFINE(new_camera, LID,
-	"(new-camera     name [CAMERA])\n\
-	Create a new camera with the given name (a string).  If a\n\
-	camera with that name already exists, the new object is given\n\
-	a unique name.  If CAMERA is omitted a default camera is used.")
+	"(new-camera     name [CAMERA])\n"
+	"Create a new camera with the given name (a string).  If a "
+	"camera with that name already exists, the new object is given "
+	"a unique name.  If CAMERA is omitted a default camera is used.")
 {
   DView *dv;
   int id;
@@ -946,11 +946,11 @@ LDEFINE(new_camera, LID,
 }
 
 LDEFINE(camera, LINT,
-	"(camera         CAM-ID [CAMERA])\n\
-	Specify data for CAM-ID; CAMERA is a string giving an OOGL\n\
-	camera specification.  If no camera CAM-ID exists,\n\
-	it is created; in this case, the second argument is optional,\n\
-	and if omitted, a default camera is used.  See also: new-camera.")
+	"(camera CAM-ID [CAMERA])\n"
+	"Specify data for CAM-ID; CAMERA is a string giving an OOGL "
+	"camera specification.  If no camera CAM-ID exists, "
+	"it is created; in this case, the second argument is optional, "
+	"and if omitted, a default camera is used.  See also: new-camera.")
 {
   DView *dv;
   int index;
@@ -992,8 +992,8 @@ LDEFINE(camera, LINT,
 }
 
 LDEFINE(camera_reset, LVOID,
-	"(camera-reset   CAM-ID)\n\
-	Reset CAM-ID to its default value.")
+	"(camera-reset CAM-ID)\n"
+	"Reset CAM-ID to its default value.")
 {
   DView *dv;
   int index;
@@ -1045,17 +1045,17 @@ drawer_new_dgeom(char *name, GeomStruct *gs, enum citizenship citizenship)
 }
 
 LDEFINE(new_alien, LID,
-	"(new-alien      name [GEOMETRY])\n\
-	Create a new alien (geom not in the world) with the given name\n\
-	(a string).  GEOMETRY is a string giving an OOGL geometry\n\
-	specification.  If GEOMETRY is omitted, the new alien\n\
-	is given an empty geometry.  If an object with that name\n\
-	already exists, the new alien is given a unique name.  The\n\
-	light beams that are used to move around the lights are an\n\
-	example of aliens. They're drawn but are not controllable the\n\
-	way ordinary objects are: they don't appear in the object\n\
-	browser and the user can't move them with the normal motion\n\
-	modes.")
+	"(new-alien      name [GEOMETRY])\n"
+	"Create a new alien (geom not in the world) with the given name "
+	"(a string).  GEOMETRY is a string giving an OOGL geometry "
+	"specification.  If GEOMETRY is omitted, the new alien "
+	"is given an empty geometry.  If an object with that name "
+	"already exists, the new alien is given a unique name.  The "
+	"light beams that are used to move around the lights are an "
+	"example of aliens. They're drawn but are not controllable the "
+	"way ordinary objects are: they don't appear in the object "
+	"browser and the user can't move them with the normal motion "
+	"modes.")
 {
   GeomStruct *gs = &nullgs;
   char *name;
@@ -1224,7 +1224,8 @@ LDEFINE(copy, LVOID,
   /* If the user copies the world, this will crash unless we remember not
    * to copy things that are copies of something else.  That is what this 
    * array is for. */
-  char *iscopy;
+  VARARRAY(iscopy, char, 2*dgeom_max);
+
   LDECLARE(("copy", LBEGIN,
 	    LOPTIONAL,
 	    LID, &id,
@@ -1234,7 +1235,6 @@ LDEFINE(copy, LVOID,
   id = real_id(id);
   geomstruct.h = camstruct.h = transformstruct.h = apstruct.h = NULL;
 
-  iscopy = alloca(2*dgeom_max*sizeof(char));
   memset(iscopy, 0, 2*dgeom_max * sizeof(char));
 
   MAYBE_LOOP(id, i, T_NONE, DObject, obj) {
@@ -1518,7 +1518,7 @@ LDEFINE(stereowin, LLIST,
 	``window'' to set the pixel aspect ratio & window position if needed.")
 {
   int id;
-  Keyword kw = NO_KEYWORD;
+  Keyword kw = NOT_A_KEYWORD;
   DView *dv;
   int gapsize = 0;
 
@@ -1533,19 +1533,23 @@ LDEFINE(stereowin, LLIST,
     OOGLError(0, "stereowin: expected camera, got %s", drawer_id2name(id));
     return Lnil;
   }
-  if(kw < 0) {
-    char *me = "stereowin";
-    LList *a = LListAppend(
-			   LListAppend( LListAppend( LListAppend(NULL, 
-								 LTOOBJ(LSTRING)(&me)),
-						     LTOOBJ(LID)(&id)),
-					LTOOBJ(LKEYWORD)(&dv->stereo)),
-			   LTOOBJ(LINT)(&dv->stereogap));
-    return LTOOBJ(LLIST)(&a);
+  if(kw == NOT_A_KEYWORD) {
+    return LLISTTOOBJ
+      ( LListAppend
+	( LListAppend
+	  ( LListAppend
+	    ( LListAppend(NULL, LSYMBOLTOOBJ("stereowin")),
+	      LTOOBJ(LID)(&id)),
+	    LTOOBJ(LKEYWORD)(&dv->stereo)),
+	  LTOOBJ(LINT)(&dv->stereogap)));
   }
   if(kw != NO_KEYWORD && kw != HORIZONTAL_KEYWORD && kw != VERTICAL_KEYWORD
      && kw != COLORED_KEYWORD) {
-    OOGLError(0, "stereowin: expected \"no\" or \"horizontal\" or \"vertical\" or \"colored\", not \"%s\"", keywordname(kw));
+    OOGLError(0, "stereowin: expected "
+	      "\"no\" or "
+	      "\"horizontal\" or "
+	      "\"vertical\" or "
+	      "\"colored\", not \"%s\"", keywordname(kw));
     return Lnil;
   }
   dv->stereo = kw;
@@ -1788,7 +1792,6 @@ drawer_int(int id, DrawerKeyword key, int ival)
   case DRAWER_TEXTUREDRAW: flag = override = APF_TEXTURE; goto setflag;
   case DRAWER_TEXTUREQUAL: flag = override = APF_TXMIPMAP|APF_TXMIPINTERP|APF_TXLINEAR; goto setflag;
   case DRAWER_BACKCULL: flag = override = APF_BACKCULL; goto setflag;
-  case DRAWER_TRANSPARENT: flag = override = APF_TRANSP; goto setflag;
   case DRAWER_EVERT: flag = override = APF_EVERT; goto setflag;
   case DRAWER_SHADELINES: flag = override = APF_SHADELINES; goto setflag;
   case DRAWER_CONCAVE: flag = override = APF_CONCAVE; goto setflag;
@@ -1797,6 +1800,13 @@ drawer_int(int id, DrawerKeyword key, int ival)
 		     AP_OVERRIDE, override & uistate.apoverride, AP_END);
     goto mergeap;
 
+  case DRAWER_TRANSLUCENCY:
+    flag = override = APF_TRANSP;
+    as.ap = ApCreate(ival < 0 ? AP_DONT : AP_DO, flag,
+		     AP_TRANSLUCENCY, abs(ival) - 1,
+		     AP_OVERRIDE, APF_TRANSP & uistate.apoverride, AP_END);
+    goto mergeap;
+
   case DRAWER_SHADING:
     as.ap = ApCreate(AP_SHADING, ival,
 		     AP_OVERRIDE, APF_SHADING & uistate.apoverride, AP_END);
@@ -2244,6 +2254,12 @@ static void delete_geometry(DGeom *dg)
   if(dgeom[index] == NULL)	/* Already (being) deleted -- forget it */
     return;
 
+  if (dg->bbox_center != NOID) {
+    /* Delete the bbox-center object */
+    gv_delete(dg->bbox_center);
+    dg->bbox_center = NOID;
+  }
+
   dgeom[index] = NULL;
   switch (dg->citizenship) {
   case ORDINARY:
@@ -2360,7 +2376,8 @@ drawer_init(char *apdefault, char *defaultcam, char *windefault)
   fsa_install(name_fsa, "targetcam",	(void *)TARGETCAMID);
   fsa_install(name_fsa, "targetcamera",	(void *)TARGETCAMID);
   fsa_install(name_fsa, "c.",		(void *)TARGETCAMID);
-  fsa_install(name_fsa, "center",	(void *)CENTERID); 
+  fsa_install(name_fsa, "center",	(void *)CENTERID);
+  fsa_install(name_fsa, "bbox-center",  (void *)BBOXCENTERID);
   fsa_install(name_fsa, "allgeoms",	(void *)ALLGEOMS);
   fsa_install(name_fsa, "g*",		(void *)ALLGEOMS);
   fsa_install(name_fsa, "allcams",	(void *)ALLCAMS);
@@ -2523,6 +2540,7 @@ drawer_init_dgeom(DGeom *dg, int id, enum citizenship citizenship)
   dg->seqno = 0;
   dg->NDT = NULL;
   dg->NDTinv = NULL;
+  dg->bbox_center = NOID;
 #if 0
   GeomSet(dg->Item, CR_NDAXIS, dg->NDT, CR_END);
 #endif
@@ -2562,6 +2580,11 @@ drawer_make_bbox(DGeom *dg, bool combine)
       GeomReplace(dg->Lbbox, bbox);
       GeomDelete(bbox);		/* Only Lbbox needs it now */
       dg->bboxvalid = true;
+      if (dg->bbox_center != NOID) {
+	/* Delete the old center object */
+	gv_delete(dg->bbox_center);
+	dg->bbox_center = NOID;
+      }
     }
   }
 }
@@ -3599,7 +3622,7 @@ LDEFINE(write_comments, LVOID,
   /* get to the right starting place */
   GeomGet(((DGeom *)drawer_get_object(id))->Lgeom, CR_GEOM, &where);
   fprintf(PoolOutputFile(op), "( ");
-  traverse(op,where,pickpath,curpath-1,curpath,pn);
+  traverse(op, where, pickpath, curpath-1, curpath, pn);
   fprintf(PoolOutputFile(op), ")\n");
   fflush(PoolOutputFile(op));
   if (temppool) {
