@@ -208,17 +208,19 @@ LDEFINE(pick_invisible, LVOID,
 	makes them invisible; default yes.\n\
 	With no arguments, returns current status.")
 {
-  Keyword kw = NO_KEYWORD;
+  Keyword kw = NOT_A_KEYWORD;
 
   LDECLARE(("pick-invisible", LBEGIN,
 	    LOPTIONAL,
 	    LKEYWORD, &kw,
 	    LEND));
 
-  if(kw < 0)
+  if(kw == NOT_A_KEYWORD) {
     return uistate.pick_invisible ? Lt : Lnil;
+  }
 
   uistate.pick_invisible = boolval("pick_invisible", kw);
+
   return Lt;
 }
   
@@ -248,7 +250,7 @@ LDEFINE(rawevent, LVOID,
 	    LINT, &event.val,
 	    LINT, &event.x,
 	    LINT, &event.y,
-	    LULONG, &event.t,
+	    LLONG, &event.t,
 	    LEND));
 
   PRINT_EVENT(("in gv_rawevent", &event));
@@ -420,7 +422,7 @@ LDEFINE(rawevent, LVOID,
 
     mote:
       id = retarget(NOID);
-      if (k) gv_ui_target( id, IMMEDIATE);
+      if (id) gv_ui_target( id, IMMEDIATE);
       gv_event_mode( s );
       break;
 
@@ -1222,7 +1224,7 @@ emit_pick(int pickedid, Pick *pick)
       /*  T = transform converting to the coord system of coordsysid */
       /* This section does the setup for the total hack */    
       Transform T;
-      float got[4];
+      HPoint3 got;
       HPoint3 v, e[2];
       HPoint3 *f;
       int gn, vn, vi, en, ei[2], ein, fn, fi;
@@ -1244,8 +1246,8 @@ emit_pick(int pickedid, Pick *pick)
 	}
 
 	if (pickedid != NOID) {
-	  Pt3Transform(T, &pick->got, (Point3 *)(void *)got);
-	  got[3] = 1;
+	  Pt3Transform(T, &pick->got, HPoint3Point3(&got));
+	  got.w = 1;
 	  gn = 4;
 	} else {
 	  gn = 0;
@@ -1285,7 +1287,7 @@ emit_pick(int pickedid, Pick *pick)
 	 * This CANNOT be called once for every interested party - otherwise
 	 * every interested party will hear about it numerous times. */
 	gv_pick(coordsysid, pickedid,
-		got, gn,
+		HPoint3Data(&got), gn,
 		&v.x, vn,
 		&e[0].x, en,
 		(float *)f, fn, 	/* f, fn, */
