@@ -56,6 +56,12 @@ char *shades[5] = {
 	"[4as] VCflat",	        /* APF_VCFLAT */
 };
 
+char *translucencies[3] = {
+  "[0mt] BSP-Tree Alpha Blending",
+  "[1mt] Screen Door",
+  "[2mt] Simple Alpha Blending"
+};
+
 char *norm[4] = {
 	"[0N] None",		/* NONE */
 	"[1N] Individual",	/* EACH */
@@ -285,6 +291,8 @@ set_ui_center_origin(int use_bbox_center)
   uistate.bbox_center = use_bbox_center != 0; /* boolean value */
   if (uistate.bbox_center && uistate.targetid != WORLDGEOM) {
     make_center_from_bbox("CENTER", uistate.targetid);
+  } else {
+    gv_ui_center(TARGETID);
   }
 }
 
@@ -373,14 +381,15 @@ LDEFINE(ap_override, LVOID,
 	Selects whether appearance controls should override objects' own\n\
 	settings.  On by default.  With no arguments, returns current setting.")
 {
-  Keyword kw = NO_KEYWORD;
+  Keyword kw = NOT_A_KEYWORD;
 
   LDECLARE(("ap-override", LBEGIN,
 	LOPTIONAL,
 	LKEYWORD, &kw,
 	LEND));
-  if(kw < 0) {
-    int on = uistate.apoverride ? ON_KEYWORD : OFF_KEYWORD;
+
+  if (kw == NOT_A_KEYWORD) {
+    Keyword on = uistate.apoverride ? ON_KEYWORD : OFF_KEYWORD;
     return LNew(LKEYWORD, &on);
   }
   drawer_int(WORLDGEOM, DRAWER_APOVERRIDE, boolval("ap-override", kw));
