@@ -127,7 +127,9 @@ extern Pixmap   geomicon;
 /*****************************************************************************/
 
 static void ui_delete_camera(Widget, XtPointer, XmAnyCallbackStruct *);
+#if MGOPENGL
 static Colormap getcmapfor(XVisualInfo *vi);
+#endif
 
 /*****************************************************************************/
 enum buffering { SGL=0, DBL=1, BUFTYPES };
@@ -302,9 +304,11 @@ Widget ui_create_camera(Widget parent, DView *dv)
   Atom   AProtocol;
   Widget shell, camform, camdraw[2] = { NULL, NULL };
   int i;
+#if MGOPENGL
   XSetWindowAttributes xswa;
   Window cmw[32];
   int cmwneeded;
+#endif
 
 #if MGOPENGL
   struct oglstuff {
@@ -946,6 +950,8 @@ ui_ppmglxsnapshot(char *fname, int id, DView *dv, WnWindow *wn, WnPosition *wp)
   int mgspace;
   mgshadefunc shader = NULL;
   int failed = 1;
+  double osfactor = 1.0;
+  char *osscale;
 
   if (gv_no_opengl) {
     OOGLError(0,
@@ -959,8 +965,18 @@ ui_ppmglxsnapshot(char *fname, int id, DView *dv, WnWindow *wn, WnPosition *wp)
     return -1;
   }
 
+  if ((osscale = getenv("GEOMVIEW_OFFSCREEN_FACTOR")) != NULL) {
+    osfactor = strtod(osscale, NULL);
+    if (osfactor == 0.0) {
+      osfactor = 1.0;
+    }
+  }
+
   xsize = wp->xmax - wp->xmin + 1;
   ysize = wp->ymax - wp->ymin + 1;
+  
+  xsize = (int)(osfactor * (double)xsize);
+  ysize = (int)(osfactor * (double)ysize);
 
   mgctxget(MG_CAMERA, &cam);
   mgctxget(MG_SPACE, &mgspace);
@@ -972,9 +988,9 @@ ui_ppmglxsnapshot(char *fname, int id, DView *dv, WnWindow *wn, WnPosition *wp)
 
   win = WnCopy(win);
   vp.xmin = vp.ymin = 0;
-  vp.xmax = xsize;
-  vp.ymax = ysize;
-  WnSet(win, WN_CURPOS, wp, WN_VIEWPORT, &vp, WN_END);
+  vp.xmax = xsize - 1;
+  vp.ymax = ysize - 1;
+  WnSet(win, WN_CURPOS, osfactor != 1.0 ? &vp : wp, WN_VIEWPORT, &vp, WN_END);
 
   /* Create an X11 pixmap and a GLX pixmap to render to */
   pixmap = XCreatePixmap(dpy, cw->shellwin, xsize, ysize, cw->vi[SGL]->depth);
@@ -1122,9 +1138,9 @@ snap_entry snapshot_table[] = {
 void
 ui_init_snaphelp()
 {
-  LHelpRedef("snapshot",
-	     "(snapshot       CAM-ID     FILENAME [FORMAT [XSIZE [YSIZE]]])\n"
-	     "Save a snapshot of CAM-ID in the FILENAME (a string). The FORMAT \
+  LHelpDef("snapshot",
+	   "(snapshot       CAM-ID     FILENAME [FORMAT [XSIZE [YSIZE]]])\n"
+	   "Save a snapshot of CAM-ID in the FILENAME (a string). The FORMAT \
 argument is optional; it may be \"ppmscreen\" (the default), \"ps\",	\
 \"ppm\", \"sgi\" (on SGI machines), \"ppmosmesa\" (if built with	\
 libOSMesa) or \"ppmosglx\".  A \"ppmscreen\" snapshot is created by	\
@@ -1297,7 +1313,10 @@ LDEFINE(bgimagefile, LSTRING,
 {
   char *fname = NULL;
   int camid;
+#if MGOPENGL || MGGL
   DView *dv;
+#endif
+
   LDECLARE(("background-image", LBEGIN,
 	    LID, &camid,
 	    LOPTIONAL,
