@@ -36,25 +36,25 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #define M_E 2.71828182845904
 #endif
 
-void fcomplex_re(fcomplex *s,fcomplex *op)
+void fcomplex_re(const fcomplex *s,fcomplex *op)
 {
   op->real = s->real;
   op->imag = 0;
 }
 
-void fcomplex_im(fcomplex *s,fcomplex *op)
+void fcomplex_im(const fcomplex *s,fcomplex *op)
 {
   op->real = 0;
   op->imag = s->imag;
 }
 
-void fcomplex_abs(fcomplex *s,fcomplex *op)
+void fcomplex_abs(const fcomplex *s,fcomplex *op)
 {
   op->imag = 0;
   op->real = sqrt(s->real*s->real+s->imag*s->imag);
 }
 
-void fcomplex_log(fcomplex *s,fcomplex *op)
+void fcomplex_log(const fcomplex *s,fcomplex *op)
 {
   /* e^(a + bi)= e^a(cos b+i sin b).
      b is argument of s, e^a is abs of s. */
@@ -62,7 +62,7 @@ void fcomplex_log(fcomplex *s,fcomplex *op)
   op->imag = atan2(s->imag,s->real);
 }
 
-void fcomplex_log10(fcomplex *s,fcomplex *op)
+void fcomplex_log10(const fcomplex *s,fcomplex *op)
 {
   /* log10(z) = log(z)/log(10) */
   fcomplex_log(s,op);
@@ -73,7 +73,7 @@ void fcomplex_log10(fcomplex *s,fcomplex *op)
   op->imag /= M_LN10;
 }
 
-void fcomplex_sqrt(fcomplex *s,fcomplex *op)
+void fcomplex_sqrt(const fcomplex *s,fcomplex *op)
 {
   fcomplex tmp;
   tmp.real = 0.5;
@@ -81,7 +81,7 @@ void fcomplex_sqrt(fcomplex *s,fcomplex *op)
   fcomplex_pow(s,&tmp,op);
 }
 
-void fcomplex_exp(fcomplex *s,fcomplex *op)
+void fcomplex_exp(const fcomplex *s,fcomplex *op)
 {
   fcomplex tmp;
   tmp.real = M_E;
@@ -89,7 +89,7 @@ void fcomplex_exp(fcomplex *s,fcomplex *op)
   fcomplex_pow(&tmp,s,op);
 }
 
-void fcomplex_cos(fcomplex *s,fcomplex *op)
+void fcomplex_cos(const fcomplex *s,fcomplex *op)
 {
   /* cos(a+bi) = cos(a)cos(bi) - sin(a)sin(bi)
      = cos(a)cosh(b) - i sin(a)sinh(b) */
@@ -97,7 +97,7 @@ void fcomplex_cos(fcomplex *s,fcomplex *op)
   op->imag = - sin(s->real) * sinh(s->imag);
 }
 
-void fcomplex_arccos(fcomplex *s,fcomplex *op)
+void fcomplex_arccos(const fcomplex *s,fcomplex *op)
 {
   /* if w = cos z,
      z = -i*ln(w+sqrt(w*w-1)) */
@@ -118,7 +118,7 @@ void fcomplex_arccos(fcomplex *s,fcomplex *op)
   op->real = tmp1.imag;
 }
 
-void fcomplex_sin(fcomplex *s,fcomplex *op)
+void fcomplex_sin(const fcomplex *s,fcomplex *op)
 {
   /* sin(a+bi) = sin(a)cos(bi) + sin(bi)cos(a)
      = sin(a)cosh(b) + isinh(b)cos(a) */
@@ -126,7 +126,7 @@ void fcomplex_sin(fcomplex *s,fcomplex *op)
   op->imag = cos(s->real)*sinh(s->imag);
 }
 
-void fcomplex_arcsin(fcomplex *s,fcomplex *op)
+void fcomplex_arcsin(const fcomplex *s,fcomplex *op)
 {
   /* if w = sin z,
      z = -i*ln(i*w+sqrt(i*(w*w-1))) */
@@ -148,7 +148,7 @@ void fcomplex_arcsin(fcomplex *s,fcomplex *op)
   op->real = tmp1.imag;
 }
 
-void fcomplex_tan(fcomplex *s,fcomplex *op)
+void fcomplex_tan(const fcomplex *s,fcomplex *op)
 {
   /* tan a+bi= Sin[2a]/(Cos[2a]+Cosh[2b]) + i*Sinh[2b]/(Cos[2a]+Cosh[2b]) */
   /* or so says mathematica(probably a registered trademark of wolfram) */
@@ -156,25 +156,25 @@ void fcomplex_tan(fcomplex *s,fcomplex *op)
   op->imag = sinh(s->imag*2)/(cos(s->real*2)+cosh(s->imag*2));
 }
 
-void fcomplex_arctan(fcomplex *s,fcomplex *op)
+void fcomplex_arctan(const fcomplex *s,fcomplex *op)
 {
   /* if w = tan z,
      z = i ln(1-iw)/2 */
-  fcomplex tmp1,tmp2;
+  fcomplex tmp1, tmp2;
 
   /* tmp1 = 1-iw */
   tmp1.real = 1+s->imag;
   tmp1.imag = -s->real;
 
   /* tmp2 = ln(tmp1); */
-  fcomplex_log(&tmp2, &tmp1);
+  fcomplex_log(&tmp1, &tmp2);
   
   /* op = i*tmp2/2 */
   op->real = -tmp2.imag/2;
   op->imag = tmp2.real/2;
 }
 
-void fcomplex_cot(fcomplex *s,fcomplex *op)
+void fcomplex_cot(const fcomplex *s,fcomplex *op)
 {
   /* cot = 1 / tan */
   fcomplex tmp1;
@@ -186,7 +186,7 @@ void fcomplex_cot(fcomplex *s,fcomplex *op)
   op->imag = -tmp1.imag/(tmp1.real*tmp1.real+tmp1.imag*tmp1.imag);
 }
 
-void fcomplex_sec(fcomplex *s,fcomplex *op)
+void fcomplex_sec(const fcomplex *s,fcomplex *op)
 {
   fcomplex tmp1;
 
@@ -195,7 +195,7 @@ void fcomplex_sec(fcomplex *s,fcomplex *op)
   op->imag = -tmp1.imag/(tmp1.real*tmp1.real+tmp1.imag*tmp1.imag);
 }
 
-void fcomplex_csc(fcomplex *s,fcomplex *op)
+void fcomplex_csc(const fcomplex *s,fcomplex *op)
 {
   fcomplex tmp1;
   fcomplex_sin(s,&tmp1);
@@ -203,7 +203,7 @@ void fcomplex_csc(fcomplex *s,fcomplex *op)
   op->imag = -tmp1.imag/(tmp1.real*tmp1.real+tmp1.imag*tmp1.imag);
 }
 
-void fcomplex_cosh(fcomplex *s,fcomplex *op)
+void fcomplex_cosh(const fcomplex *s,fcomplex *op)
 {
   /* cosh[a + b i] = cos[b - ai] */
   fcomplex tmp1;
@@ -212,7 +212,7 @@ void fcomplex_cosh(fcomplex *s,fcomplex *op)
   fcomplex_cos(&tmp1,op);
 }
 
-void fcomplex_sinh(fcomplex *s,fcomplex *op)
+void fcomplex_sinh(const fcomplex *s,fcomplex *op)
 {
   /* sinh [a + b i] = i * sin[ b - a i] */
   fcomplex tmp1,tmp2;
@@ -224,7 +224,7 @@ void fcomplex_sinh(fcomplex *s,fcomplex *op)
   op->imag = tmp2.real;
 }
 
-void fcomplex_tanh(fcomplex *s,fcomplex *op)
+void fcomplex_tanh(const fcomplex *s,fcomplex *op)
 {
   /* tanh[a+bi] = i*tan[b - a i] */
   fcomplex tmp1,tmp2;
@@ -236,7 +236,7 @@ void fcomplex_tanh(fcomplex *s,fcomplex *op)
   op->imag = tmp2.real;
 }
 
-void fcomplex_arccosh(fcomplex *s,fcomplex *op)
+void fcomplex_arccosh(const fcomplex *s,fcomplex *op)
 {
   /* z=cos[iw]=cosh[w]
      arccosh[z] = w = arccos[z]/i
@@ -247,8 +247,10 @@ void fcomplex_arccosh(fcomplex *s,fcomplex *op)
   op->imag = -tmp1.real;
 }
 
-void fcomplex_arcsinh(fcomplex *s,fcomplex *op)
+void fcomplex_arcsinh(const fcomplex *s,fcomplex *op)
 {
+  (void)op;
+
   /* zi=sin[iw]=isinh[w]
      arcsinh[z] = w = arcsin[zi]/i 
      arcsinh[z] = arcsin[zi]/i */
@@ -262,10 +264,12 @@ void fcomplex_arcsinh(fcomplex *s,fcomplex *op)
   tmp2.imag = -tmp1.real;
 }
 
-void fcomplex_arctanh(fcomplex *s,fcomplex *op)
+void fcomplex_arctanh(const fcomplex *s,fcomplex *op)
 {
   /* works same as arcsinh.. */
   fcomplex tmp1,tmp2;
+
+  (void)op;
   /* tmp1 = i*z */
   tmp1.real = -s->imag;
   tmp1.imag = s->real;
@@ -275,26 +279,26 @@ void fcomplex_arctanh(fcomplex *s,fcomplex *op)
   tmp2.imag = -tmp1.real;
 }
 
-void fcomplex_floor(fcomplex *s,fcomplex *op)
+void fcomplex_floor(const fcomplex *s,fcomplex *op)
 {
   /* just floor the components, i guess... */
   op->real = floor(s->real);
   op->imag = floor(s->imag);
 }
 
-void fcomplex_round(fcomplex *s,fcomplex *op)
+void fcomplex_round(const fcomplex *s,fcomplex *op)
 {
   op->real = rint(s->real);
   op->imag = rint(s->imag);
 }
 
-void fcomplex_ceiling(fcomplex *s,fcomplex *op)
+void fcomplex_ceiling(const fcomplex *s,fcomplex *op)
 {
   op->real = ceil(s->real);
   op->imag = ceil(s->imag);
 }
 
-void fcomplex_pow(fcomplex *base,fcomplex *exponent,fcomplex*op)
+void fcomplex_pow(const fcomplex *base, const fcomplex *exponent, fcomplex*op)
 {
   double Abs, Arg;
   
@@ -332,7 +336,7 @@ sin(a+bi) = cos(a)*i*sinh(b) + cosh(b)*sin(a)
   Power(E,d*Arg)
 */
 
-void fcomplex_atan2(fcomplex *y, fcomplex *x, fcomplex *op)
+void fcomplex_atan2(const fcomplex *y, const fcomplex *x, fcomplex *op)
 {
   /* mathematica says: -i log((x+yi)/sqrt(x^2+y^2)),
      so i'm going to do:
@@ -341,7 +345,8 @@ void fcomplex_atan2(fcomplex *y, fcomplex *x, fcomplex *op)
 
   /* tmp1 = x^2 + y^2 */
   tmp1.real=x->real*x->real-x->imag*x->imag+y->real*y->real-y->imag*y->imag;
-  
+  tmp1.imag=0;
+
   /* tmp2 = log(tmp1) */
   fcomplex_log(&tmp1,&tmp2);
 
@@ -359,12 +364,14 @@ void fcomplex_atan2(fcomplex *y, fcomplex *x, fcomplex *op)
 
 /* stubs */
 
-void fcomplex_monstub(fcomplex *arg, fcomplex *op)
+void fcomplex_monstub(const fcomplex *arg, fcomplex *op)
 {
+  (void)arg;
   op->real = op->imag = atof("NaN");
 }
 
-void fcomplex_binstub(fcomplex *arg1, fcomplex *arg2, fcomplex *op)
+void fcomplex_binstub(const fcomplex *arg1, const fcomplex *arg2, fcomplex *op)
 {
+  (void)arg1;
   op->real = op->imag = atof("NaN");
 }
