@@ -31,7 +31,9 @@
 #include "plane.h"
 #include "line.h"
 #include "point3.h"
+#include "hpointn.h"
 #include "transform.h"
+#include "transformn.h"
 #include "radians.h"
 #include "ooglutil.h"
 #include "polyint.h"
