@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 
 /* Authors: Charlie Gunn, Pat Hanrahan, Stuart Levy, Tamara Munzner, Mark Phillips */
@@ -32,15 +34,27 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include <stdio.h>
 #include "transform3.h"
 
-void
-Tm3Print( T )
-    Transform3 T;
+void Tm3Print(FILE *f, Transform3 T)
 {
     int i, j;
 
-    printf( "[" );
-    for( i=0; i<4; i++ )
-	for( j=0; j<4; j++ )
-	    printf( "%g ", T[i][j] );
-    printf( "]\n" );
+    if (f == NULL) {
+	return;
+    }
+    fprintf(f, "transform {\n");
+    for (i=0; i<4; i++) {
+	fprintf(f, "  ");
+	for (j=0; j<4; j++) {
+	    fprintf(f, "%10.7f ", T[i][j] );
+	}
+	fprintf(f, "\n" );
+    }
+    fprintf(f, "}\n" );
 }
+
+/*
+ * Local Variables: ***
+ * mode: c ***
+ * c-basic-offset: 4 ***
+ * End: ***
+ */
