@@ -33,6 +33,23 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include "bboxP.h"
 #include "appearance.h"
 
+DEF_FREELIST(BBox);
+
+void BBoxFreeListPrune(void)
+{
+  FreeListNode *old;
+  size_t size = 0;
+
+  HPtNFreeListPrune();
+  while (BBoxFreeList) {
+    old = BBoxFreeList;
+    BBoxFreeList = old->next;
+    OOGLFree(old);
+    size += sizeof(BBox);
+  }
+  OOGLWarn("Freed %ld bytes.\n", size);
+}
+
 #ifndef max
 # define max(a,b) ((a) > (b) ? (a) : (b))
 #endif
@@ -50,12 +67,12 @@ int BBoxGet(BBox *bbox, int attr, void *attrp)
   case CR_MIN:
     HPtNToHPt3(bbox->min, NULL, &min3);
     HPt3Dehomogenize(&min3, &min3);
-    *(Point3 *)attrp = *(Point3 *)(void *)&min3;
+    memcpy(attrp, &min3, sizeof(Point3));
     break;
   case CR_MAX:
     HPtNToHPt3(bbox->max, NULL, &max3);
     HPt3Dehomogenize(&max3, &max3);
-    *(Point3 *)attrp = *(Point3 *)(void *)&max3;
+    memcpy(attrp, &max3, sizeof(Point3));
     break;
   case CR_4MIN: HPtNToHPt3(bbox->min, NULL, (HPoint3 *)attrp); break;
   case CR_4MAX: HPtNToHPt3(bbox->max, NULL, (HPoint3 *)attrp); break;
@@ -91,11 +108,12 @@ BBox *BBoxCreate (BBox *exist, GeomClass *classp, va_list *a_list)
   int attr, copy = 1, need_update = 0;
 
   if (exist == NULL) {
-    bbox = OOGLNewE(BBox, "BBoxCreate BBox");
+    FREELIST_NEW(BBox, bbox);
+    GGeomInit(bbox, classp, BBOXMAGIC, NULL);
+    bbox->freelisthead = &BBoxFreeList;
     bbox->min = HPtNCreate(4, NULL);
     bbox->max = HPtNCreate(4, NULL);
     bbox->center = HPtNCreate(4, NULL);
-    GGeomInit (bbox, classp, BBOXMAGIC, NULL);
   } else {
     /* FIXME: Check that exist is in fact a BBox. */
     bbox = exist;
@@ -154,12 +172,12 @@ BBox *BBoxCreate (BBox *exist, GeomClass *classp, va_list *a_list)
       maxNp = need_update = 1;
       break;
     default:
-      if (GeomDecorate (bbox, &copy, attr, a_list)) {
+      if (GeomDecorate (bbox, &copy, attr, a_list) && exist == NULL) {
 	OOGLError(0, "BBoxCreate: Undefined attribute: %d", attr);
 	HPtNDelete(bbox->min);
 	HPtNDelete(bbox->max);
 	HPtNDelete(bbox->center);
-	OOGLFree(bbox);
+	FREELIST_FREE(BBox, bbox);
 	return NULL;
       }
     }
@@ -223,10 +241,12 @@ BBox *BBoxCreate (BBox *exist, GeomClass *classp, va_list *a_list)
   return bbox;
 
  error:
-  HPtNDelete(bbox->min);
-  HPtNDelete(bbox->max);
-  HPtNDelete(bbox->center);
-  OOGLFree(bbox);
+  if (exist == NULL) {
+    HPtNDelete(bbox->min);
+    HPtNDelete(bbox->max);
+    HPtNDelete(bbox->center);
+    FREELIST_FREE(BBox, bbox);
+  }
   return NULL;
 }
 
