@@ -80,6 +80,11 @@ static int tree_depth;
 				 * on our obstack
 				 */
 
+void BSPTreeNodeDataFreeListPrune(void)
+{
+  NodeDataFreeListPrune();
+}
+
 static void check_poly(Poly *poly)
 {
   int i;
@@ -300,6 +305,9 @@ QuadToLinkedPolyList(Transform T, Transform Tdual, Transform TxT,
   Poly *qpoly;
   int i, j, concave;
 
+  (void)Tdual;
+  (void)TxT;
+
   if (!plistp) {
     plistp = &plist;
   }
@@ -705,7 +713,7 @@ PolyListToLinkedPoyList(Transform T, Transform Tdual, Transform TxT,
       ListPush(*plistp, new_pn);
       break;
 #if !HAVE_LIBGLU
-case 4: /* supported */
+    case 4: /* supported */
       if (pl->p[pnr].flags & (POLY_NONFLAT|POLY_CONCAVE)) {
 	/* split this polygon along a diagonal, if the polygon is
 	 * concave: split across the unique concave vertex.
@@ -760,6 +768,7 @@ case 4: /* supported */
 	new_pn->poly = poly;
 	ListPush(*plistp, new_pn);
       }
+      break;
       /* otherwise fall into the > 4 vertices case and leave
        * everything to the GLU tesselator.
        */
@@ -976,6 +985,7 @@ void BSPTreeFreeTree(const BSPTree *_tree)
  */
 void BSPTreeInvalidate(Handle **chp, Ref *unused, BSPTree *self)
 {
+  (void)unused;
   BSPTreeFreeTree(self);
 }
 
@@ -1122,8 +1132,9 @@ static inline void SplitPolyNode(PolyListNode *plnode,
 				 struct obstack *scratch)
 {
   const void **tagged_app = plnode->tagged_app;
-  Poly *poly, savedp;
-  Vertex *v0, *v1, **vpos, **savedv;
+  Poly *poly = plnode->poly, savedp;
+  VARARRAY(savedv, Vertex *, poly->n_vertices);
+  Vertex *v0, *v1, **vpos;
   int istart[2], iend[2], i, nv[2];
   Vertex *vstart[2], *vend[2];
 
@@ -1131,8 +1142,6 @@ static inline void SplitPolyNode(PolyListNode *plnode,
   ++n_tree_polys;
 #endif
 
-  poly = plnode->poly;
-
   vstart[0] = vstart[1] = vend[0] = vend[1] = NULL;
   istart[0] = istart[1] = iend[0] = iend[1] = -1;
 
@@ -1295,8 +1304,8 @@ static inline void SplitPolyNode(PolyListNode *plnode,
   }
 
   if (poly->flags & POLY_SCRATCH) {
+
     savedp = *poly;
-    savedv = alloca(poly->n_vertices*sizeof(Vertex *));
     memcpy(savedv, poly->v, poly->n_vertices*sizeof(Vertex *));
 
     if (nv[0] <= poly->n_vertices) {
