@@ -19,25 +19,34 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 
 /* Authors: Charlie Gunn, Stuart Levy, Tamara Munzner, Mark Phillips */
 
 #include "geomclass.h"
+#include "nodedata.h"
 
-void
-GeomReplace( parent, newchild )
-    Geom *parent;
-    Geom *newchild;
+void GeomReplace(Geom *parent, Geom *newchild)
 {
     if(parent != NULL && parent->Class->replace != NULL) {
 	RefIncr((Ref *)newchild);
 	GeomDelete((*parent->Class->replace)(parent, newchild));
+
+	GeomNodeDataPrune(parent);
     }
 }
+
+/*
+ * Local Variables: ***
+ * mode: c ***
+ * c-basic-offset: 4 ***
+ * End: ***
+ */
