@@ -25,9 +25,9 @@
 #ifndef VERTDEFS
 #define VERTDEFS
 
-#define VERT_P 0
-#define VERT_N 1
-#define VERT_C 2
-#define VERT_4D 4
+#define VERT_P  0
+#define VERT_N  (1 << 0)
+#define VERT_C  (1 << 1)
+#define VERT_4D (1 << 2)
 
 #endif
