@@ -45,6 +45,8 @@ NPolyListFSave(NPolyList *pl, FILE *outf, char *fname)
   HPtNCoord *v;
   ColorA *c;
 
+  (void)fname;
+
   /* We don't really know the number of edges and it's a pain to count.
    * Assume Euler number 2.
    */
@@ -99,7 +101,7 @@ NPolyListFSave(NPolyList *pl, FILE *outf, char *fname)
     fprintf(outf, "\n%d	", p->n_vertices);
     for(k = 0; k < p->n_vertices; k++)
       fprintf(outf, " %d", pl->vi[pl->pv[i] + k]);
-    if((pl->geomflags & (PL_HASPCOL|PL_HASVCOL)) == PL_HASPCOL) {
+    if((pl->geomflags & PL_HASPCOL)) {
       fprintf(outf, "\t%.8g %.8g %.8g %.8g",
 	      p->pcol.r, p->pcol.g, p->pcol.b, p->pcol.a);
     }
