@@ -38,15 +38,14 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include "polylistP.h"
 
 PolyList *
-PolyListFSave(polylist, outf, fname)
-	PolyList *polylist;	/* Thing to save */
-	FILE *outf;			/* Stream to save it on */
-	char *fname;			/* File name for error msgs */
+PolyListFSave(PolyList *polylist, FILE *outf, char *fname)
 {
 	int i, n;
 	Poly *p;
 	Vertex **vp, *v;
 
+	(void)fname;
+
 	/* We don't really know the number of edges and it's a pain to count.
 	 * Assume Euler number 2.
 	 */
@@ -79,9 +78,9 @@ PolyListFSave(polylist, outf, fname)
 	    fprintf(outf, "\n%d	", n);
 	    for(vp = p->v; --n >= 0; vp++)
 		    fprintf(outf, " %d", (int)((*vp) - polylist->vl));
-	    if((polylist->geomflags & (PL_HASPCOL|PL_HASVCOL)) == PL_HASPCOL) {
-		fprintf(outf, "\t%g %g %g %g",
-			p->pcol.r, p->pcol.g, p->pcol.b, p->pcol.a);
+	    if ((polylist->geomflags & PL_HASPCOL)) {
+	      fprintf(outf, "\t%g %g %g %g",
+		      p->pcol.r, p->pcol.g, p->pcol.b, p->pcol.a);
 	    }
 	}
 	fputc('\n', outf);
