@@ -35,28 +35,58 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
  * Tlist creation, editing and deletion
  */
 
-void
-TlistDelete( Tlist *tlist )
+DEF_FREELIST(Tlist);
+
+void TlistFreeListPrune(void)
+{
+  FreeListNode *old;
+  Tlist *oldtl;
+  size_t size = 0;
+  
+  while (TlistFreeList) {
+    old = TlistFreeList;
+    TlistFreeList = old->next;
+    oldtl = (Tlist *)old;
+    if (oldtl->elements != NULL && oldtl->nallocated != 0) {
+      OOGLFree(oldtl->elements);
+    }
+    size += sizeof(Transform) * oldtl->nallocated;
+    OOGLFree(old);
+    size += sizeof(Tlist);
+  }
+  OOGLWarn("Freed %ld bytes.\n", size);
+}
+
+void TlistDelete(Tlist *tlist)
 {
     if(tlist) {
-	if(tlist->tlist != NULL) GeomDelete(tlist->tlist);
-	if(tlist->tlisthandle != NULL) HandlePDelete(&tlist->tlisthandle);
-	if(tlist->elements != NULL) OOGLFree(tlist->elements);
+	if (tlist->tlist != NULL) GeomDelete(tlist->tlist);
+	if (tlist->tlisthandle != NULL) HandlePDelete(&tlist->tlisthandle);
+	if (tlist->elements != NULL && tlist->freelisthead == NULL) {
+	  OOGLFree(tlist->elements);
+	  tlist->elements = NULL;
+	  tlist->nallocated = 0;
+	}
     }
 }
 
-Tlist *
-TlistCopy( Tlist *t) 
+Tlist *TlistCopy(Tlist *t) 
 {
   Tlist *nt;
   int i;
 
-  nt = OOGLNewE(Tlist, "TlistCopy: Tlist");
+  FREELIST_NEW(Tlist, nt);
   GGeomInit(nt, t->Class, t->magic, NULL);
+  nt->freelisthead = &TlistFreeList;
   nt->nelements = t->nelements;
-  nt->elements = OOGLNewNE(Transform, nt->nelements, "TList transforms");
-  for (i = 0; i < nt->nelements; i++)
+  if (nt->nallocated < nt->nelements) {
+    nt->elements =
+      OOGLRenewNE(Transform, nt->elements, nt->nelements, "Tlist transforms");
+    nt->nallocated = nt->nelements;
+  }
+  for (i = 0; i < nt->nelements; i++) {
     TmCopy(t->elements[i], nt->elements[i]);
+  }
   nt->tlist = GeomCopy(t->tlist);
   nt->tlisthandle = NULL;
   return(nt);
@@ -92,19 +122,20 @@ TlistCreate(Tlist *exist, GeomClass *Classp, va_list *a_list)
     Tlist *tlist;
     Geom *g;
     Handle *h;
-    int copy = 1;
-    int copyel = 0;
+    int  copy   = true;
+    bool copyel = false;
+    bool doinit = false;
     int attr;
     int i;
-    Transform *elements = (Transform *) NULL;
+    Transform *elements = NULL;
 
     if (exist == NULL) {
-	tlist = OOGLNewE(Tlist, "TlistCreate tlist");
+	FREELIST_NEW(Tlist, tlist);
+	GGeomInit (tlist, Classp, TLISTMAGIC, NULL);
+	tlist->freelisthead = &TlistFreeList;
 	tlist->tlisthandle = NULL;
 	tlist->tlist = NULL;
 	tlist->nelements = 0;
-	tlist->elements = NULL;
-	GGeomInit (tlist, Classp, TLISTMAGIC, NULL);
     } else {
 	/* XXX Check that exist is a tlist. */
 	tlist = exist;
@@ -119,12 +150,20 @@ TlistCreate(Tlist *exist, GeomClass *Classp, va_list *a_list)
 	    h = va_arg(*a_list, Handle *);
 	  settlist:
 	    g = va_arg(*a_list, Geom *);
-	    if(copy) RefIncr((Ref *)g);
-	    if(tlist->tlist) GeomDelete(tlist->tlist);
+	    if(copy) {
+		RefIncr((Ref *)g);
+	    }
+	    if(tlist->tlist) {
+		GeomDelete(tlist->tlist);
+	    }
 	    tlist->tlist = g;
 	  sethandle:
-	    if(tlist->tlisthandle) HandleDelete(tlist->tlisthandle);
-	    if(copy && h) RefIncr((Ref *)h);
+	    if(tlist->tlisthandle) {
+		HandleDelete(tlist->tlisthandle);
+	    }
+	    if(copy && h) {
+		RefIncr((Ref *)h);
+	    }
 	    tlist->tlisthandle = h;
 	    break;
 	case CR_TLISTHANDLE:
@@ -133,35 +172,52 @@ TlistCreate(Tlist *exist, GeomClass *Classp, va_list *a_list)
 	case CR_ELEM:
 	    elements = va_arg (*a_list, Transform *);
 	    copyel = copy;
+	    if (!copyel) {
+		OOGLFree(tlist->elements);
+		tlist->elements   = NULL;
+		tlist->nallocated = 0;
+	    }
+	    if (elements == NULL) {
+	      doinit = true;
+	    }
 	    break;
 	case CR_NELEM:
 	    tlist->nelements = va_arg (*a_list, int);
-	    if (tlist->elements != NULL) {
-		OOGLFree (tlist->elements);
-		tlist->elements = NULL;
-	    }
+	    doinit = true;
 	    break;
 	default:
 	    if(GeomDecorate(tlist, &copy, attr, a_list)) {
 		OOGLError (0, "TlistCreate: undefined option: %d", attr);
-		if (exist == NULL) GeomDelete ((Geom *) tlist);
+		if (exist == NULL) {
+		    GeomDelete ((Geom *) tlist);
+		}
 		return NULL;
 	    }
     }
-
-    if ((tlist->elements == NULL) && (tlist->nelements > 0)) {
-	if (elements != NULL && !copyel)
-	    tlist->elements = elements;
-	else {
-	    tlist->elements = OOGLNewNE(Transform, tlist->nelements,
-				"TlistCreate: matrices");
-	    if (elements == NULL) 
-		for (i=0; i<tlist->nelements; i++)
-		    TmIdentity (tlist->elements[i] );
-	    else
-		memcpy (tlist->elements, elements,
-				tlist->nelements * sizeof (Transform));
-	}    
+    
+    if (elements != NULL && !copyel) {
+      if (tlist->elements) {
+	OOGLFree(tlist->elements);
+      }
+      tlist->elements   = elements;
+      tlist->nallocated = tlist->nelements;
+    } else {
+      if (tlist->nallocated < tlist->nelements) {
+	tlist->elements =
+	  OOGLRenewNE(Transform, tlist->elements, tlist->nelements,
+		      "TlistCreate: matrices");
+	tlist->nallocated = tlist->nelements;
+      }
+      if (elements == NULL) {
+	if (doinit) {
+	  for (i = 0; i < tlist->nelements; i++) {
+	    TmIdentity (tlist->elements[i] );
+	  }
+	}
+      } else {
+	memcpy (tlist->elements, elements,
+		tlist->nelements * sizeof (Transform));
+      }
     }
 
     return tlist;
