@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 #include "mgP.h"
 #include "mgps.h"
@@ -37,7 +39,7 @@ void        mgps_reshapeviewport(void);
 void        mgps_appearance( struct mgastk *ma, int mask );
 void        mgps_setshader(mgshadefunc shader);
 mgcontext * mgps_ctxcreate(int a1, ...);
-void        mgps_ctxset( int a1, ...  );
+int         mgps_ctxset( int a1, ...  );
 int         mgps_feature( int feature );
 void        mgps_ctxdelete( mgcontext *ctx );
 int         mgps_ctxget( int attr, void* valueptr );
@@ -53,8 +55,7 @@ void        mgps_gettransform( Transform T );
 void        mgps_settransform( Transform T );
 int         mgps_pushappearance( void );
 int         mgps_popappearance( void );
-Appearance *mgps_setappearance( Appearance* app, int merge );
-Appearance *mgps_getappearance( void );
+const Appearance *mgps_setappearance(const Appearance* app, int merge );
 int         mgps_setcamera( Camera* cam );
 int         mgps_setwindow( WnWindow *win, int final );
 mgpscontext *mgps_newcontext( mgpscontext *ctx );
@@ -69,7 +70,7 @@ extern void mgps_polyline();
 extern void mgps_polylist();
 extern void mgps_quads();
 
-void _mgps_ctxset(int a1, va_list *alist);
+int _mgps_ctxset(int a1, va_list *alist);
 
 WnWindow *mgpswindow(WnWindow *win);
 
@@ -79,23 +80,33 @@ struct mgfuncs mgpsfuncs = {
   mgps_feature,
   (mgcontext *(*)())mgps_ctxcreate,
   mgps_ctxdelete,
-  (void (*)())mgps_ctxset,
+  (int (*)())mgps_ctxset,
   mgps_ctxget,
   mgps_ctxselect,
   mgps_sync,
   mgps_worldbegin,
   mgps_worldend,
   mgps_reshapeviewport,
+  /* geometry transform */
   mgps_settransform,
   mgps_gettransform,
   mgps_identity,
   mgps_transform,
   mgps_pushtransform,
   mgps_poptransform,
+  /* texture geometry transform */
+  mg_settxtransform,
+  mg_gettxtransform,
+  mg_txidentity,
+  mg_txtransform,
+  mg_pushtxtransform,
+  mg_poptxtransform,
+  /* appearances */
   mgps_pushappearance,
   mgps_popappearance,
   mgps_setappearance,
-  mgps_getappearance,
+  mg_getappearance,
+  /***************/
   mgps_setcamera,
   mgps_polygon,
   mgps_polylist,
@@ -103,7 +114,11 @@ struct mgfuncs mgpsfuncs = {
   mgps_line,
   mgps_polyline,
   mg_quads,
-  mg_bezier
+  mg_bezier,
+  mg_bsptree,
+  mg_tagappearance,
+  mg_untagappearance,
+  mg_taggedappearance
   };
 
 
@@ -163,16 +178,15 @@ mgps_newcontext( mgpscontext *ctx )
  * Description: internal ctxset routine
  * Args:        a1: first attribute
  *              *alist: rest of attribute-value list
- * Returns:     nothing
+ * Returns:     -1 on error, 0 on success
  * Date:        Fri Sep 20 11:08:13 1991
  * Notes:       mgps_ctxcreate() and mgps_ctxset() call this to actually
  *              parse and interpret the attribute list.
  */
-void
+int
 _mgps_ctxset(int a1, va_list *alist)
 {
   int attr;
-  WnWindow *owin;
   char **ablock = NULL;
 
 #define NEXT(type) OOGL_VA_ARG(type, alist, ablock)
@@ -287,12 +301,8 @@ _mgps_ctxset(int a1, va_list *alist)
       }
       break;
 
-    case MG_NDINFO:
-      _mgc->NDinfo = NEXT(void *);
-      break;
-
-    case MG_NDMAP:
-      _mgc->NDmap = NEXT(mgmapfunc);
+    case MG_NDCTX:
+      _mgc->NDctx = NEXT(mgNDctx *);
       break;
 
     case MG_ZNUDGE:
@@ -311,7 +321,7 @@ _mgps_ctxset(int a1, va_list *alist)
 
     default:
       OOGLError (0, "_mgps_ctxset: undefined option: %d", attr);
-      return;
+      return -1;
     }
   }
   if (_mgc->shown && !_mgpsc->born) {
@@ -330,6 +340,7 @@ _mgps_ctxset(int a1, va_list *alist)
 
 #undef NEXT
 
+  return 0;
 }
 
 
@@ -338,12 +349,7 @@ _mgps_ctxset(int a1, va_list *alist)
 int
 mgps_setwindow( WnWindow *win, int final )
 {
-  WnPosition wp;
-  WnPosition pos, vp;
-  int xsize, ysize, flag, reconstrain;
-  int positioned = 0;
-  int zmin;
-  char *name, *oname;
+  int xsize, ysize;
 
   if (win == NULL)
    return 0;
@@ -459,8 +465,8 @@ mgps_pushappearance()
 int
 mgps_popappearance()
 {
-  register struct mgastk *mastk = _mgc->astk;
-  register struct mgastk *mastk_next;
+  struct mgastk *mastk = _mgc->astk;
+  struct mgastk *mastk_next;
 
   if (!(mastk_next = mastk->next))
   {
@@ -488,10 +494,10 @@ mgps_popappearance()
  *		than mgps_setappearance currently does???  This
  *		seems common to all devices.
  */
-Appearance *
-mgps_setappearance( Appearance *ap, int mergeflag )
+const Appearance *
+mgps_setappearance(const Appearance *ap, int mergeflag )
 {
-  int changed, mat_changed, lng_changed;
+  int changed, lng_changed;
   struct mgastk *mastk = _mgc->astk;
   Appearance *ma;
 
@@ -500,16 +506,13 @@ mgps_setappearance( Appearance *ap, int mergeflag )
   if(mergeflag == MG_MERGE)
   {
     changed = ap->valid &~ (ma->override &~ ap->override);
-    mat_changed =
-    ap->mat ? ap->mat->valid &~ (ma->mat->override &~ ap->mat->override) : 0;
     lng_changed =
-    ap->lighting ? ap->lighting->valid &~
-    (ma->lighting->override &~ ap->lighting->override) : 0;
+	ap->lighting ? ap->lighting->valid &~
+	(ma->lighting->override &~ ap->lighting->override) : 0;
   }
   else
   {
     changed = ap->valid;
-    mat_changed = ap->mat ? ap->mat->valid : 0;
     lng_changed = ap->lighting ? ap->lighting->valid : 0;
   }
   mg_setappearance( ap, mergeflag );
@@ -525,24 +528,6 @@ mgps_setappearance( Appearance *ap, int mergeflag )
   return ap;
 }
 
-/*-----------------------------------------------------------------------
- * Function:	mgps_getappearance
- * Description:	get the current appearance
- * Returns:	ptr to the current appearance
- * DEVICE USE:	optional
- * Notes:	The pointer returned points to the context's private copy
- *		of the appearance.  Don't modify it!
- *
- *		Should we allow this?  Or should this copy the appearance
- *		to an address passed as an argument ???
- */
-Appearance *
-mgps_getappearance()
-{
-  return( &(_mgc->astk->ap) );
-}
-
-
 /*-----------------------------------------------------------------------
  * Function:	mgps_setcamera
  * Description:	Set the context's camera
@@ -567,21 +552,23 @@ mgps_setcamera( Camera *cam )
  * Description:	set some attributes in the current context
  * Args:	attr, ...: list of attribute-value pairs, terminated
  *		  by MG_END
- * Returns:	nothing
+ * Returns:	-1 on error, 0 on success
  * Notes:	DO NOT CALL THIS (yet)!  It currently does nothing.
  * DEVICE USE:  forbidden --- devices have their own mgxx_ctxset()
  *
  *		This needs to be modified to work as the NULL device.
  *		Use by other devices may never be needed.
  */
-void
+int
 mgps_ctxset( int attr, ... /*, MG_END */ )
 {
   va_list alist;
+  int result;
 
   va_start( alist, attr );
-  _mgps_ctxset(attr, &alist);
+  result = _mgps_ctxset(attr, &alist);
   va_end(alist);
+  return result;
 }
 
 
@@ -637,12 +624,8 @@ mgps_ctxget( int attr, void *value )
       *VALUE(int) = _mgc->space;
       break;
 
-    case MG_NDINFO:
-      *VALUE(void *) = _mgc->NDinfo;
-      break;
-
-    case MG_NDMAP:
-      *VALUE(mgmapfunc) = _mgc->NDmap;
+    case MG_NDCTX:
+      *VALUE(mgNDctx *) = _mgc->NDctx;
       break;
 
     case MG_ZNUDGE:
@@ -698,7 +681,9 @@ mgps_ctxcreate( int a1, ... )
     (mgcontext*)mgps_newcontext( OOGLNewE(mgpscontext, "mgps_ctxcreate") );
   mgps_initpsdevice();
   va_start(alist, a1);
-  _mgps_ctxset(a1, &alist);
+  if (_mgps_ctxset(a1, &alist) == -1) {
+      mgps_ctxdelete(_mgc);
+  }
   va_end(alist);
   return _mgc;
 }
@@ -779,9 +764,6 @@ mgps_sync( void )
 void
 mgps_worldbegin( void )
 {
-  Transform S;
-  WnPosition vp;
-  float aspect;
 
   mg_worldbegin();
 
@@ -792,9 +774,6 @@ mgps_worldbegin( void )
 void
 mgps_worldend( void )
 {
-    int i, size;
-    FILE *file;
-
     if (_mgpsc->file == NULL)
 	return;
     mgps_sortdisplaylist();
@@ -816,10 +795,21 @@ void
 mgps_setshader(mgshadefunc shader)
 {
     struct mgastk *ma = _mgc->astk;
-    int wasusing = ma->useshader;
+    unsigned short wasusing = ma->flags & MGASTK_SHADER;    
 
     ma->shader = shader;
-    ma->useshader = (shader != NULL && IS_SHADED(ma->ap.shading)) ? 1 : 0;
-    if(ma->useshader != wasusing)
+    if (shader != NULL && IS_SHADED(ma->ap.shading)) {
+	ma->flags |= MGASTK_SHADER;
+    } else {
+	ma->flags &= ~MGASTK_SHADER;
+    }
+    if((ma->flags & MGASTK_SHADER) != wasusing)
 	mgps_appearance(_mgc->astk, APF_SHADING);
 }
+
+/*
+ * Local Variables: ***
+ * mode: c ***
+ * c-basic-offset: 4 ***
+ * End: ***
+ */
