@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 #include "mgP.h"
 #include "mgpsP.h"
@@ -36,8 +38,8 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 
 void	mgps_polygon( int nv, HPoint3 *v, int nn, Point3 *n,
 	      	          int nc, ColorA *c );
-void	mgps_mesh( int wrap, int nu, int nv, HPoint3 *p, Point3 *n,
-			ColorA *c );
+void	mgps_mesh(int wrap, int nu, int nv, HPoint3 *p, Point3 *n, Point3 *nq,
+		  ColorA *c );
 void	mgps_line( HPoint3 *p1, HPoint3 *p2 );
 void	mgps_polyline( int nv, HPoint3 *verts, int nc, ColorA *colors,
 			int wrap );
@@ -62,17 +64,16 @@ mgps_polygon(int nv,  HPoint3 *V,
 	     int nn,  Point3 *N, 
 	     int nc,  ColorA *C)
 {
-  register int		count;
-  register HPoint3     *v;
-  register Point3      *n;
-  int			flag, cinc, ninc, smooth;
+  int		count;
+  HPoint3     *v;
+  Point3      *n;
+  int			flag, ninc, smooth;
 
 /*  fprintf(stderr,"X11: draw a polygon\n"); */
 
   flag = _mgc->astk->ap.flag;
   if ((_mgc->astk->mat.override & MTF_DIFFUSE) && 
-	!_mgc->astk->useshader) nc = 0;
-  cinc = (nc > 1);
+      !(_mgc->astk->flags & MGASTK_SHADER)) nc = 0;
   ninc = (nn > 1);
   smooth = IS_SMOOTH(_mgc->astk->ap.shading);
 /*
@@ -169,11 +170,10 @@ void mgps_line( HPoint3 *p1, HPoint3 *p2 )
  * Notes:	See mg.doc.
  *
  */
-void mgps_fatpoint(register HPoint3 *v)
+void mgps_fatpoint(HPoint3 *v)
 {
-  int i;
   HPoint3 a;
-  register HPoint3 *p, *q;
+  HPoint3 *p, *q;
   float vw;
 
 /*  fprintf(stderr,"X11: make a fat-point\n"); */
@@ -291,10 +291,9 @@ void mgps_polyline( int nv, HPoint3 *v, int nc, ColorA *c, int wrapped )
  */
 void mgps_polylist( int np, Poly *_p, int nv, Vertex *V, int pl_flags )
 {
-  register int		i, j;
-  register Poly		*p;
-  register Vertex 	**v, *vp, **vh;
-  register Point3	*n;
+  int		i, j;
+  Poly		*p;
+  Vertex 	**v, *vp, **vh;
   struct mgastk *ma = _mgc->astk;
   int plflags = pl_flags;
   int flag, shading;
@@ -305,21 +304,20 @@ void mgps_polylist( int np, Poly *_p, int nv, Vertex *V, int pl_flags )
 
 /*  fprintf(stderr,"X11: draw a polylist %d\n",np); */
 
-  switch(shading)
-  {
-    case APF_FLAT:
-      plflags &= ~PL_HASVN;
-      break;
-    case APF_SMOOTH:
-      plflags &= ~PL_HASPN;
-      break;
-    default:
-      plflags &= ~(PL_HASVN | PL_HASPN);
-      break;
+  switch(shading) {
+  case APF_FLAT:
+    plflags &= ~PL_HASVN;
+    if (plflags & PL_HASPCOL) {
+      plflags &= ~PL_HASVCOL;
+    }
+    break;
+  case APF_SMOOTH: plflags &= ~PL_HASPN; break;
+  case APF_VCFLAT: plflags &= ~PL_HASVN; break;
+  default: plflags &= ~(PL_HASVN|PL_HASPN); break;
   }
 
   if ((_mgc->astk->mat.override & MTF_DIFFUSE) &&
-	!_mgc->astk->useshader)
+      !(_mgc->astk->flags & MGASTK_SHADER))
     plflags &= ~(PL_HASVCOL | PL_HASPCOL);
 
   if (flag & APF_FACEDRAW)
@@ -592,16 +590,13 @@ void mgpspolymeshrow(int wrap, int has, int off, int count, HPoint3 *P,
 void mgpssubmesh(int wrap, int nu, int nv, int umin, int umax, int vmin,
 		int vmax, HPoint3 *meshP, Point3 *meshN, ColorA *meshC)
 {
-  register int u, v;
-  int ucnt, vcnt;
-  register HPoint3 *P;
-  register Point3 *N;
-  register ColorA *C;
-  register int prev;
+  int v;
+  int ucnt;
+  HPoint3 *P;
+  Point3 *N;
+  ColorA *C;
+  int prev;
   int du;
-  int douwrap;
-  HPoint3  tp;
-  Point3 tn;
   int i;
   int has;
   Appearance *ap;
@@ -612,11 +607,11 @@ void mgpssubmesh(int wrap, int nu, int nv, int umin, int umax, int vmin,
 
   ap = &_mgc->astk->ap;
   if ((_mgc->astk->mat.override & MTF_DIFFUSE) &&
-	!_mgc->astk->useshader)
+      !(_mgc->astk->flags & MGASTK_SHADER))
     meshC = 0;
 
   has = 0;
-  if (meshN && !_mgc->astk->useshader)
+  if (meshN && !(_mgc->astk->flags & MGASTK_SHADER))
     has = HAS_N;
   if (meshC)
     has |= HAS_C;
@@ -680,8 +675,8 @@ void mgpssubmesh(int wrap, int nu, int nv, int umin, int umax, int vmin,
  * Notes:	See mg.doc.
  *
  */
-void mgps_mesh( int wrap, int nu, int nv, HPoint3 *P, Point3 *N,
-			ColorA *C )
+void mgps_mesh(int wrap, int nu, int nv, HPoint3 *P, Point3 *N, Point3 *NQ,
+	       ColorA *C )
 {
   mgpssubmesh( wrap, nu, nv, 0, nu-1, 0, nv-1, P, N, C);
 }
@@ -725,28 +720,33 @@ come to think of it, what is the correct interpretation of the
 normal vector when the points live in 4-d?
 */
 void
-mgps_drawnormal(register HPoint3 *p, Point3 *n)
+mgps_drawnormal(HPoint3 *p, Point3 *n)
 {
   Point3 tp;
   HPoint3 end;
-  float scale;
+  Pt3Coord scale, w, s;
 
 /*  fprintf(stderr,"X11: draw a normal\n"); */
   if (p->w <= 0.0) return;
   if (p->w != 1) {
     HPt3ToPt3(p, &tp);
-    p = (HPoint3 *)&tp;
+    p = (HPoint3 *)(void *)&tp;
   }
 
   scale = _mgc->astk->ap.nscale;
-  if (_mgc->astk->ap.flag & APF_EVERT)
-  {
-    register Point3 *cp = &_mgc->cpos;
-    if (!(_mgc->has & HAS_CPOS))
+  if (_mgc->astk->ap.flag & APF_EVERT) {
+    HPoint3 *cp = &_mgc->cpos;
+    if (!(_mgc->has & HAS_CPOS)) {
       mg_findcam();
-    if ((p->x - cp->x) * n->x + (p->y - cp->y) * n->y +
-		(p->z - cp->z) * n->z > 0)
+    }
+    if ((w = cp->w) != 1.0 && w != 0.0) {
+      s = (p->x*w-cp->x)*n->x + (p->y*w-cp->y)*n->y + (p->z*w-cp->z)*n->z;
+    } else {
+      s = (p->x-cp->x)*n->x + (p->y-cp->y)*n->y + (p->z-cp->z)*n->z;
+    }
+    if (s > 0) {
       scale = -scale;
+    }
   }
   end.x = p->x + scale*n->x;
   end.y = p->y + scale*n->y;
