@@ -115,6 +115,11 @@ extern Handle *HandleByName( char *, HandleOps * );
 	 */
 extern Handle *HandleCreate( char *name, HandleOps *ops );
 
+	/*
+	 * Unconditionally create a new Handle with NULL value.
+	 */
+extern Handle *HandleDoCreate( char *name, HandleOps *ops );
+
 	/*
 	 * Same as HandleCreate() for an existing global handle, but
 	 * if the handle does not already exist increment the
