@@ -96,7 +96,7 @@ static fd_set poolreadyfds;
 static int poolnready = 0;
 
 #define	FOREVER  ((((unsigned)1)<<31)-1)
-static struct timeval nexttowake = { FOREVER };
+static struct timeval nexttowake = { FOREVER, };
 
 static Pool *newPool(char *name);
 
@@ -141,7 +141,7 @@ HandleReferringTo(int prefixch, char *str, HandleOps *ops, Handle **hp)
     } else {			/*   <  file:name */
 	name = sep+1;
 	fname = nb;
-	if (sep-str >= sizeof(nb)) {
+	if ((size_t)(sep-str) >= sizeof(nb)) {
 	    sep = &str[sizeof(nb)-1];
 	}
 	memcpy(fname, str, sep-str);
@@ -182,6 +182,7 @@ HandleReferringTo(int prefixch, char *str, HandleOps *ops, Handle **hp)
 	} else {
 	    h = ph; /* otherwise we return the handle returned by PoolIn(). */
 	}
+	HandleDelete(hknown);
     } else if (p) {
 	/* If we have a pool but not handle, then generate one for
 	 * this pool; callers of this functinos treat handle == NULL
@@ -204,8 +205,6 @@ HandleReferringTo(int prefixch, char *str, HandleOps *ops, Handle **hp)
 	}
     }
 
-    HandleDelete(hknown);
-
     if (hp) {
 	if (*hp) {
 	    if (*hp != h) {
@@ -240,6 +239,22 @@ PoolByName(char *fname, HandleOps *ops)
     return NULL;
 }
 
+void pool_dump(void)
+{
+    Pool *p;
+    Handle *h;
+
+    OOGLWarn("Active Pools:");
+    DblListIterateNoDelete(&AllPools, Pool, node, p) {
+	OOGLWarn("  %s[%s]%p",
+		 p->ops ? p->ops->prefix : "none",
+		 p->poolname, (void *)p);
+	OOGLWarn("    Attached Handles:");
+	DblListIterateNoDelete(&p->handles, Handle, poolnode, h) {
+	    OOGLWarn("    %s", h->name);
+	}
+    }
+}
 
 static
 void watchfd(int fd)
@@ -483,8 +498,9 @@ PoolStreamOpen(char *name, FILE *f, int rw, HandleOps *ops)
 	if (rw != 1) {
 	    p->inf = iobfileopen(f);
 	}
-	if (rw > 0)
+	if (rw > 0) {
 	    p->outf = (rw == 2) ? fdopen(dup(fileno(f)), "wb") : f;
+	}
     }
 
     if (p->inf == NULL && p->outf == NULL) {
@@ -564,6 +580,7 @@ PoolIncLevel(Pool *p, int incr)
 int
 PoolOType(Pool *p, int otype)
 {
+    (void)otype;
     return p->otype;
 }
 
@@ -605,10 +622,11 @@ PoolDoReread(Pool *p)
 
 void PoolClose(Pool *p)
 {
-    if (p->ops->close && !(p->flags & PF_CLOSING)) {
+    if (p->ops->close && (p->flags & PF_CLOSING) == 0) {
 	p->flags |= PF_CLOSING;
-	if ((*p->ops->close)(p))
+	if ((*p->ops->close)(p)) {
 	    return;
+	}
     }
 
     if (p->type == P_STREAM) {
@@ -626,7 +644,7 @@ void PoolClose(Pool *p)
 	    if (p->outf != stdout) fclose(p->outf);
 	    p->outf = NULL;
 	}
-	/* PoolDelete(p);*/ /* Shouldn't we delete the Pool? */
+	/* PoolDelete(p); */ /* Shouldn't we delete the Pool? */
     }
 }
 
@@ -637,6 +655,7 @@ void PoolDelete(Pool *p)
     if (p == NULL || (p->flags & PF_DELETED) != 0) {
 	return;
     }
+
     p->flags |= PF_DELETED;
 
     if ((p->flags & PF_TEMP) == 0) {
@@ -652,6 +671,22 @@ void PoolDelete(Pool *p)
     FREELIST_FREE(Pool, p);
 }
 
+void PoolDetach(Pool *p)
+{
+    if ((p->flags & PF_TEMP) == 0) {
+	DblListDelete(&p->node);
+    }
+}
+
+void PoolReattach(Pool *p)
+{
+    if ((p->flags & PF_TEMP) == 0) {
+	if (DblListEmpty(&p->node)) {
+	    DblListAddTail(&AllPools, &p->node);
+	}
+    }
+}
+
 /*
  * Marks a Pool as being awake.  Doesn't update nexttowake;
  * so this should only be called where awaken_until() gets a chance to run.
@@ -727,13 +762,20 @@ PoolInputFDs(fd_set *fds, int *maxfd)
  * Each fd used is removed from *fds, and *nfds is decremented.
  * The return value is 1 if anything was read from any pool, 0 otherwise.
  */
+
 int
 PoolInAll(fd_set *fds, int *nfds)
 {
-    Pool *p, *nextp;
+    Pool *p;
     int got = 0;
 
-    DblListIterate(&AllPools, Pool, node, p, nextp) {
+    /* We use DblListIterateNoDelete() _NOT_ because PoolIn() could
+     * not delete the current Pool p, because PoolIn() can delete
+     * _ANY_ pool so we need to work harder to make this
+     * PoolInAll()-loop failsafe.
+     */
+
+    DblListIterateNoDelete(&AllPools, Pool, node, p) {
 	if (p->type != P_STREAM || p->inf == NULL || p->infd < 0) {
 	    continue;
 	}
@@ -751,34 +793,27 @@ PoolInAll(fd_set *fds, int *nfds)
 		got++;
 	    }
 	}
+	/* We need to be very careful here: PoolIn() can have _ANY_
+	 * sort of side effect, in particular it might delete _ANY_
+	 * pool. Luckily we do not really delete pools; if "p" is
+	 * actually in the deleted state, then it has its PF_DELETED
+	 * flag set. In this case we simply restart the loop.
+	 *
+	 * NOTE: Just using &AllPools triggers a strict aliasing
+	 * warning with gcc. Of course, I don't know if the funny
+	 * "next->prev" stuff simply confuses the compiler such that
+	 * it does no longer emit the warning, but still emits wrong
+	 * code. In principle the construct should be ok. "next->prev"
+	 * should always point back to &AllPools. Of course, the
+	 * compiler cannot know this. Maybe this is the difference.
+	 */
+	if (p->flags & PF_DELETED) {
+	    p = DblListContainer(AllPools.next->prev, Pool, node);
+	}
     }
     return got;
 }
 
-/*
- * Handle NULL or uninitialized times.
- */
-static struct timeval *
-timeof(struct timeval *when)
-{
-   static struct timeval now;
-   if ((when == NULL && (when = &now)) || !timerisset(when))
-	gettimeofday(when, NULL);
-   return when;
-}
-    
-static void
-addtime(struct timeval *base, double offset, struct timeval *result)
-{
-    double osec = floor(offset);
-    result->tv_sec = base->tv_sec + osec;
-    result->tv_usec = base->tv_usec + (int)((offset - osec)*1000000);
-    while(result->tv_usec >= 1000000) {
-	result->tv_sec++;
-	result->tv_usec -= 1000000;
-    }
-}
-
 static void
 asleep(Pool *p, struct timeval *base, double offset)
 {
@@ -787,9 +822,11 @@ asleep(Pool *p, struct timeval *base, double offset)
     base = timeof(base);
     if (p->inf != NULL) {
 	p->flags |= PF_ASLEEP;
-	addtime(base, offset, &until);
-	if (timercmp(&until, &nexttowake, <))
+	addtime(&until, base, offset);
+	if (timercmp(&until, &nexttowake, <)) {
 	    nexttowake = until;
+	}
+	p->awaken = until;
 	if (p->infd >= 0) {
 	    unwatchfd(p->infd);
 	    if (FD_ISSET(p->infd, &poolreadyfds)) {
@@ -816,7 +853,7 @@ void
 PoolSetTime(Pool *p, struct timeval *base, double time_at_base)
 {
     base = timeof(base);
-    addtime(base, -time_at_base, &p->timebase);
+    addtime(&p->timebase, base, -time_at_base);
 }
 
 double
@@ -836,6 +873,20 @@ PoolAwaken(Pool *p)
 	awaken_until(&nexttowake);
 }
 
+bool
+PoolASleep(Pool *p)
+{
+    if (p->flags & PF_ASLEEP) {
+	struct timeval now;
+	gettimeofday(&now, NULL);
+	if (timercmp(&p->awaken, &now, >)) {
+	    return true;
+	}
+	awaken(p);
+    }
+    return false;
+}
+
 #if 0
 static void
 poolresync(Pool *p, int (*resync)())
@@ -878,10 +929,12 @@ Handle *PoolIn(Pool *p)
 		     * ownership of the object to the Handle.
 		     */
 		    REFPUT(r);
-		    /* Increment the reference count lest PoolDelete()
-		     * will also consume the attached object.
-		     */
-		    REFGET(Handle, h);
+		    if (h->whence == NULL) {
+			/* Increment the reference count lest PoolDelete()
+			 * will also consume the attached object.
+			 */
+			REFGET(Handle, h);
+		    }
 		}
 	    } else {
 		/* Increment the count such that the calling function
@@ -910,6 +963,9 @@ Handle *PoolIn(Pool *p)
 	     * containing multiple objects, so we actually do reread if asked.
 	     */
 	    p->flags |= (p->flags & PF_ANY) ? PF_REREAD : PF_ANY;
+
+	    /* handle_dump(); */
+
 	} else {
 	    if (p->flags & PF_DELETED)
 		return NULL;
@@ -917,7 +973,7 @@ Handle *PoolIn(Pool *p)
 		(*p->ops->resync)(p);
 	    } else if (p->softEOF) {
 		iobfrewind(p->inf);
-	    } else if (p->inf != NULL) {	/* Careful lest already PoolClose()d */
+	    } else if (p->inf != NULL) { /* Careful lest already PoolClose()d */
 		if (p->infd >= 0) {
 		    if (FD_ISSET(p->infd, &poolreadyfds)) {
 			FD_CLR(p->infd, &poolreadyfds);
@@ -948,6 +1004,8 @@ Handle *PoolIn(Pool *p)
 		 */
 	} else {
 	    PoolClose(p);
+	    REFGET(Handle, h); /* PoolDelete() deletes also the handle. */
+	    PoolDelete(p);
 	    return h;
 	}
     }
