@@ -135,6 +135,7 @@ extern void PoolSleepUntil(Pool *, double seconds);
 extern void PoolSetTime(Pool *, struct timeval *basis, double time_at_basis);
 extern double PoolTimeAt(Pool *, struct timeval *at_this_time);
 extern void PoolAwaken(Pool *);
+extern bool PoolASleep(Pool *p);
 
 extern int PoolIncLevel(Pool *p, int incr);
 
@@ -145,6 +146,9 @@ extern int PoolClearMark(Pool *p);
 extern void PoolFPrint(Pool *p, FILE *f, const char *format, ...);
 extern void PoolPrint(Pool *p, const char *format, ...);
 
+extern void PoolDetach(Pool *p);
+extern void PoolReattach(Pool *p);
+
 #endif /*_STREAMPOOL_*/
 
 /*
