@@ -31,7 +31,7 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 
 /* Authors: Charlie Gunn, Stuart Levy, Tamara Munzner, Mark Phillips */
 
-/* $Header: /cvsroot/geomview/geomview/src/lib/oogl/util/futil.c,v 1.18 2007/05/16 19:38:07 rotdrop Exp $ */
+/* $Header: /cvsroot/geomview/geomview/src/lib/oogl/util/futil.c,v 1.20 2010/03/13 16:41:35 rotdrop Exp $ */
 
 /*
  * Geometry object routines. These routines have their "back-seekable"
@@ -166,7 +166,9 @@ fputnf(FILE *file, int count, float *v, int binary)
 	  long w;
 	  for(i = 0; i < count; i++) {
 	    w = gv_htonl(*(long *)&v[i]);
-	    fwrite(&w, sizeof(float), 1, file);
+	    if (fwrite(&w, sizeof(float), 1, file) != 1) {
+	      return i;
+	    }
 	  }
 	  return count;
 #endif
@@ -664,13 +666,13 @@ ftoken(FILE *file, int flags)
  * needed before the next call to ftoken().
  */
 char *
-fdelimtok(char *delims, FILE *file, int flags)
+fdelimtok(const char *delims, FILE *file, int flags)
 {
 	static char *token = NULL;
 	static int troom = 0;
 	int c;
 	char *p;
-	char *q;
+	const char *q;
 	int term;
 
 	if((term = fnextc(file, flags)) == EOF)
