@@ -41,17 +41,35 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include "appearance.h"
 #include "ooglutil.h"
 
-static void norm();
+static inline void
+norm(Color *color, float *coeff);
 
+DEF_FREELIST(LtLight);
+
+#if 0
 #define	NewLtLight()	OOGLNewE(LtLight, "new LtLight")
 #define	FreeLtLight(lt)	OOGLFree(lt)
+#else
+static inline LtLight* NewLtLight(void)
+{
+    LtLight *newlt;
+    
+    FREELIST_NEW(LtLight, newlt);
+    
+    return newlt;
+}
+static inline void FreeLtLight(LtLight *old)
+{
+    FREELIST_FREE(LtLight, old);
+}
+
+#endif
 
 /*
  * Default light is full white and position along the z-axis.
  */
 
 static Color black = { 0.0, 0.0, 0.0 };
-/* static LtLight *freelights = NULL; */
 
 LtLight *
 _LtSet(LtLight *light, int a1, va_list *alist)
@@ -278,12 +296,20 @@ LtFLoad(LtLight *lite, IOBFILE *f, char *fname)
 	return NULL;
       }
       switch(i) {
-      case 0: l.ambient = *(Color *)(void *)v; break;
-      case 1: l.color = *(Color *)(void *)v; 
-	norm( &l.color, &l.intensity ); break;
-      case 2: l.position = *(HPoint3 *)(void *)v; break;
-      case 3: break;
-      default: l.location = ~largs[i]; break;
+      case 0:
+	  memcpy(&l.ambient, v, sizeof(Color));
+	  break;
+      case 1:
+	  memcpy(&l.color, v, sizeof(Color));
+	  norm( &l.color, &l.intensity );
+	  break;
+      case 2:
+	  memcpy(&l.position, v, sizeof(HPoint3));
+	  break;
+      case 3:
+	  break;
+      default:
+	  l.location = ~largs[i]; break;
       }
     }
   }
@@ -520,9 +546,11 @@ void
 LmCopyLights(LmLighting *from, LmLighting *to)
 {
     int i;
-    LtLight **lp;
+    LtLight **lp, *tmp;
+
     LM_FOR_ALL_LIGHTS(from, i, lp) {
-	LmAddLight(to, *lp);
+	LmAddLight(to, tmp = LtCopy(*lp, NULL));
+	LtDelete(tmp);
     }
 }
   
@@ -612,10 +640,8 @@ LmDeleteLights(LmLighting *lm)
 # define max(a, b) ((a) > (b) ?( a) : (b))
 #endif
 
-static void
-norm( color, coeff )
-    Color *color;
-    float *coeff;
+static inline void
+norm(Color *color, float *coeff)
 {
     *coeff = max(color->r, color->g);
     *coeff = max(color->b, *coeff);
@@ -712,7 +738,7 @@ LmFLoad(LmLighting *lgt, IOBFILE *f, char *fname)
 	lgt->valid |= lbits[i];
 	if(over) lgt->override |= lbits[i];
 	switch(i) {
-	case 0: lgt->ambient = *(Color *)(void *)v; break;
+	case 0: memcpy(&lgt->ambient, v, sizeof(Color)); break;
 	case 1: lgt->localviewer = v[0]; break;
 	case 2: lgt->attenconst = v[0]; break;
 	case 3: lgt->attenmult = v[0]; break;
@@ -740,6 +766,8 @@ LmFSave(LmLighting *li, FILE *f, char *fname, Pool *p)
     LtLight **lp;
     int i;
 
+    (void)fname;
+
     PoolFPrint(p, f, "ambient %.8g %.8g %.8g\n", 
 	li->ambient.r,
 	li->ambient.g,
